/**********************************************************
 * ModServ2x - Modular IRC Services for advanced use.     *
 *   Based on the Undernet/Beware P10 Protocol.           *
 *   (C) 2004-2005 Christian Jurk (commx@commx.ws)        *
 *   Special thanks goes to:                              *
 *     Stephan Peijnik                                    *
 *     Linus Kimselius                                    *
 * ------------------------------------------------------ *
 * This program is released under the terms of the GNU    *
 * Public License (GPL). You'll find the License in the   *
 * docs directory shipped with this release.              *
 * ------------------------------------------------------ *
 * channels.c                                             *
 * Channels, users, pointers and stuff.                   *
 * 21/04/05 11:49 commx                                   *
 **********************************************************/

#include "include/main.h"
#include "include/ucdata.h"
#include "include/clients.h"
#include "include/channels.h"
#include "include/dynhdl.h"
#include "include/pgsql.h"
#include "include/configx.h"

chans *cptr(char *name) {
  chans *a;
  for(a=chan_list;a!=NULL;a=a->next) {
    if (!a->name) continue;
    if (global_debug>=6) {
      printf("cptr(): [%s][%s]\n",a->name,name);
    }
    if (!strncasecmp(a->name,name,stdc_name)) {
      return a;
    }
  }
  return NULL;
}

cusers *cuptr(char *chan, char *unum) {
  cusers *a;
  for(a=cuser_list;a!=NULL;a=a->next) {
    if (!a->name || !a->unum) continue;
    if (!strncmp(a->name,chan,stdc_name) && !strncmp(a->unum,unum,5)) {
      return a;
    }
  }
  return NULL;
}

void adduchan(char *unum, char *chan, time_t created) {
  uchans *a;
  a=(uchans *)myalloc(sizeof(uchans));
  a->join=created;
  a->name=(char*)myalloc(strlen(chan)+1);
  strncpy(a->name,chan,strlen(chan));
  a->unum=(char*)myalloc(strlen(unum)+1);
  strncpy(a->unum,unum,strlen(unum));
  a->name[strlen(chan)]='\0';
  a->unum[strlen(unum)]='\0';
  a->next=uchan_list;
  uchan_list=a;
}

void newchannel(char *chan, char *unum, time_t created) {
  chans *a; cusers *b,*b2=NULL; uchans *c,*c2=NULL;
  a=cptr(chan);
  if (a!=NULL) {
    // channel already exist, delete existing users
    for(b=cuser_list;b!=NULL;b=b->next) {
      if (!b->name) continue;
      if (!strncasecmp(b->name,chan,stdc_name)) {
        for(c=uchan_list;c!=NULL;c=c->next) {
          if (!c->name) continue;
          if (!strncasecmp(c->name,chan,stdc_name) && !strncmp(c->unum,b->unum,5)) {
            if (c2 == NULL) {
              uchan_list=c->next;
            } else {
              c2->next=c->next;
            }
            free(c->name);
            free(c->unum);
            free(c);
            break;
          }
          c2=c;
        }
        b2 = b;
        free(b->name);
        free(b->unum);
        free(b);
        b = b2;
        return;
      }
    }
  } else {
    // channel is new
    a=(chans *)myalloc(sizeof(chans));
    a->created=created;
    strcp(a->name,chan);
    a->users=(cusers *)myalloc(sizeof(cusers));
    a->users->name=(char*)myalloc(strlen(chan)+1);
    strncpy(a->users->name,chan,strlen(chan));
    a->users->unum=(char*)myalloc(strlen(unum)+1);
    strncpy(a->users->unum,unum,strlen(unum));
    a->users->name[strlen(chan)]=0x00;
    a->users->unum[strlen(unum)]=0x00;
    SetOpped(a->users->flags);
    a->users->next=cuser_list;
    cuser_list=a->users;
    strcp(a->modes,"+");
    adduchan(unum,chan,a->created);
    //addcuser(chan,unum,a->created);
    a->regops=NULL;
    a->cid=0;
    a->limit=0;
    a->joins=1;
	a->topic[0]='\0';
	a->locked=0;
    resetchar(a->key);
    a->next=chan_list;
    chan_list=a;
  }
}

void addcuser(char *chan, char *unum, time_t jts) {
  cusers *b; chans *c;
  b=(cusers *)myalloc(sizeof(cusers));
  b->name=(char*)myalloc(strlen(chan)+1);
  b->unum=(char*)myalloc(strlen(unum)+1);
  strcp(b->name,chan);
  strcp(b->unum,unum);
  c=cptr(chan);
  if (c!=NULL) {
    c->joins++;
  }
  b->flags[0]='\0';
  b->next=cuser_list;
  cuser_list=b;
}

void delcuser(char *chan, char *unum) {
  cusers *a,*a2=NULL; uchans *b,*e=NULL; chans *c,*c2=NULL; unsigned int i;
  for(a=cuser_list;a!=NULL;a=a->next) {
    if (!a->name) continue;
    if (!strncasecmp(a->name,chan,stdc_name) && !strncmp(a->unum,unum,5)) {
      if (a2 == NULL) {
        cuser_list=a->next;
      } else {
        a2->next=a->next;
      }
      free(a->name);
      free(a->unum);
      free(a);
      break;
    }
    a2 = a;
  }
  for(i=0,a=cuser_list;a!=NULL;a=a->next) {
    if (!a->name) continue;
    if (!strncmp(a->name,chan,stdc_name)) i++;
  }
  if (i==0) {
    // no users left, delete channel
    for(c=chan_list;c!=NULL;c=c->next) {
      if (!c->name) continue;
      if (!strncmp(c->name,chan,stdc_name)) {
        if (c2 == NULL) {
          chan_list=c->next;
        } else {
          c2->next = c->next;
        }
        free(c);
        break;
      }
      c2 = c;
    }
  }
}

void setmodes(chans *c, char *modestr) {
  unsigned int i; unsigned long a;
  a=0;
  if (modestr==NULL) { 
    strcp(c->modes,"+");
    return;
  }
  for(i=0;i<strlen(modestr);i++) {
    if (modestr[i]=='+') continue;
    if (modestr[i]=='c') {
      appchar(c->modes,cm_c);
      continue;
    } else if (modestr[i]=='C') {
      appchar(c->modes,cm_C);
      continue;
    } else if (modestr[i]=='d') {
      appchar(c->modes,cm_d);
      continue;
    } else if (modestr[i]=='D') {
      appchar(c->modes,cm_D);
      continue;
    } else if (modestr[i]=='i') {
      appchar(c->modes,cm_i);
      continue;
    } else if (modestr[i]=='k') {
      appchar(c->modes,cm_k);
      continue;
    } else if (modestr[i]=='l') {
      appchar(c->modes,cm_l);
      continue;
    } else if (modestr[i]=='m') {
      appchar(c->modes,cm_m);
      continue;
    } else if (modestr[i]=='M') {
      appchar(c->modes,cm_M);
      continue;
    } else if (modestr[i]=='n') {
      appchar(c->modes,cm_n);
      continue;
    } else if (modestr[i]=='N') {
      appchar(c->modes,cm_N);
      continue;
    } else if (modestr[i]=='p') {
      appchar(c->modes,cm_p);
      continue;
    } else if (modestr[i]=='r') {
      appchar(c->modes,cm_r);
      continue;
    } else if (modestr[i]=='s') {
      appchar(c->modes,cm_s);
      continue;
    } else if (modestr[i]=='S') {
      appchar(c->modes,cm_S);
      continue;
    } else if (modestr[i]=='t') {
      appchar(c->modes,cm_t);
      continue;
    } else if (modestr[i]=='T') {
      appchar(c->modes,cm_T);
      continue;
    } else if (modestr[i]=='u') {
      appchar(c->modes,cm_u);
      continue;      
    } else if (modestr[i]=='z') {
      appchar(c->modes,cm_z);
      continue;
    } else {
      slog("** Warning: Unknown mode to set: [+%c]",modestr[i]);
      continue;
    }
  }
  strsort(c->modes);
  if (global_debug) printf("c->modes=(%s)\n",c->modes);
}

void setmode(chans *c, char modestr) {
  if (global_debug) printf("setmode (%c)\n",modestr);
  if (modestr=='c') {
    appchar(c->modes,cm_c);
  } else if (modestr=='C') {
    appchar(c->modes,cm_C);
  } else if (modestr=='d') {
    appchar(c->modes,cm_d);
  } else if (modestr=='D') {
    appchar(c->modes,cm_D);
  } else if (modestr=='i') {
    appchar(c->modes,cm_i);
  } else if (modestr=='k') {
    appchar(c->modes,cm_k);
  } else if (modestr=='l') {
    appchar(c->modes,cm_l);
  } else if (modestr=='m') {
    appchar(c->modes,cm_m);
  } else if (modestr=='M') {
    appchar(c->modes,cm_M);
  } else if (modestr=='n') {
    appchar(c->modes,cm_n);
  } else if (modestr=='N') {
    appchar(c->modes,cm_N);
  } else if (modestr=='p') {
    appchar(c->modes,cm_p);
  } else if (modestr=='r') {
    appchar(c->modes,cm_r);
  } else if (modestr=='s') {
    appchar(c->modes,cm_s);
  } else if (modestr=='S') {
    appchar(c->modes,cm_S);
  } else if (modestr=='t') {
    appchar(c->modes,cm_t);
  } else if (modestr=='T') {
    appchar(c->modes,cm_T);
  } else if (modestr=='u') {
    appchar(c->modes,cm_u);    
  } else if (modestr=='z') {
    appchar(c->modes,cm_z);
  } else {
    slog("** Warning: Unknown mode to set: [+%c]",modestr);
  }
  strsort(c->modes);
  if (global_debug) printf("result: (%s)\n",c->modes);
}

void delmode(chans *c, char modestr) {
  if (modestr=='c') {
    delchar(c->modes,cm_c);
    return;
  } else if (modestr=='C') {
    delchar(c->modes,cm_C);
    return;
  } else if (modestr=='d') {
    delchar(c->modes,cm_d);
    return;
  } else if (modestr=='D') {
    delchar(c->modes,cm_D);
    return;
  } else if (modestr=='i') {
    delchar(c->modes,cm_i);
    return;
  } else if (modestr=='k') {
    delchar(c->modes,cm_k);
    return;
  } else if (modestr=='l') {
    delchar(c->modes,cm_l);
    return;
  } else if (modestr=='m') {
    delchar(c->modes,cm_m);
    return;
  } else if (modestr=='M') {
    delchar(c->modes,cm_M);
    return;
  } else if (modestr=='n') {
    delchar(c->modes,cm_n);
    return;
  } else if (modestr=='N') {
    delchar(c->modes,cm_N);
    return;
  } else if (modestr=='p') {
    delchar(c->modes,cm_p);
    return;
  } else if (modestr=='r') {
    delchar(c->modes,cm_r);
    return;
  } else if (modestr=='s') {
    delchar(c->modes,cm_s);
    return;
  } else if (modestr=='S') {
    delchar(c->modes,cm_S);
    return;
  } else if (modestr=='t') {
    delchar(c->modes,cm_t);
    return;
  } else if (modestr=='T') {
    delchar(c->modes,cm_T);
    return;
  } else if (modestr=='u') {
    delchar(c->modes,cm_u);
    return;    
  } else if (modestr=='z') {
    delchar(c->modes,cm_z);
    return;
  } else {
    slog("** Warning: Unknown mode to set: [+%c]",modestr);
  }
}

void buildcmodes(chans *c, char *modes) {
  unsigned int i; char mo[20];
  strcpy(modes,"+");
  if (c->modes==NULL) return;
  resetchar(mo);
  for(i=0;i<strlen(c->modes);i++) {
    sprintf(mo,"%c",c->modes[i]);
    strcat(modes,mo);
    resetchar(mo);
  }
  strsortn(modes);
}

char *getchanlev(char *chan, char *account) {
  PGresult *res; char *b,*c; int cid;
  cid=getcid(chan);
  strtolower(account);
  res=pg_query("SELECT flags FROM chanlevs WHERE lower(account)='%s' AND cid=%ld;",account,cid);
  if (!res || (pg_num_rows(res)<1)) {
    PQclear(res);
    return NULL;
  }
  b=PQgetvalue(res,0,0);
  if (!b || b==NULL || b[0]=='\0') { return NULL; }  
  c=(char*)(malloc(strlen(b)));
  strncpy(c,b,strlen(b));
  c[strlen(b)]=0x00;
  strsort(c); 
  return c;
}

unsigned int check4cl(char *chan, char *unum, char flag) {
  char *a,*b;
  if (!isauthed(unum)) return 0;
  a=getauth(unum);
  if (a==NULL) { return 0; }
  b=getchanlev(chan,a);
  if (b==NULL) { return 0; }
  if (chrstr(flag,b)) { return 1; } else { return 0; }
  free(b);
}  

unsigned int cl_auto(char *chan, char *unum) {
  return check4cl(chan,unum,'a');
}

unsigned int cl_banned(char *chan, char *unum) {
  return check4cl(chan,unum,'b');
}

unsigned int cl_give(char *chan, char *unum) {
  return check4cl(chan,unum,'g');
}

unsigned int cl_invite(char *chan, char *unum) {
  return check4cl(chan,unum,'i');
}

unsigned int cl_master(char *chan, char *unum) {
  return check4cl(chan,unum,'m');
}

unsigned int cl_owner(char *chan, char *unum) {
  return check4cl(chan,unum,'n');
}

unsigned int cl_op(char *chan, char *unum) {
  return check4cl(chan,unum,'o');
}

unsigned int cl_deny(char *chan, char *unum) {
  return check4cl(chan,unum,'q');
}

unsigned int cl_topic(char *chan, char *unum) {
  return check4cl(chan,unum,'t');
}

unsigned int cl_voice(char *chan, char *unum) {
  return check4cl(chan,unum,'v');
}

unsigned int cl_welcome(char *chan, char *unum) {
  return check4cl(chan,unum,'w');
}

unsigned int hascl(char *chan, char *unum) {
  if (!isauthed(unum)) return 0;
  if (!cl_auto(chan,unum) && !cl_give(chan,unum) && !cl_invite(chan,unum) && !cl_master(chan,unum) && !cl_owner(chan,unum) && !cl_op(chan,unum) && !cl_topic(chan,unum) && !cl_voice(chan,unum) && !cl_welcome(chan,unum)) { return 0; } else { return 1; }
}

unsigned int isvalidcl(char flag) {
  int i,c;
  for(c=0,i=0;i<strlen(stdflags_cl);i++) {
    if (stdflags_cl[i]==flag) {
      c=1;
    }
  }
  return c;
}

char *getcflags(char *chan) {
  PGresult *res; char *b,*c;
  strtolower(chan);
  res=pg_query("SELECT flags FROM channels WHERE lower(channel)='%s';",chan);
  if (!res || (pg_num_rows(res)<1)) {
    PQclear(res);
    return NULL;
  }
  b=PQgetvalue(res,0,0);
  c=(char*)myalloc(strlen(b)+1);
  strcp(c,b);
  PQclear(res);
  return c;
}

unsigned int check4cf(char *chan, char flag) {
  char *h;
  h=getcflags(chan);
  if (h==NULL) { return 0; }
  if (chrstr(flag,h)) {
    free(h);
    return 1;
  }
  free(h);
  return 0;
}

unsigned int isvalidcf(char flag) {
  int i,c;
  for(c=0,i=0;i<strlen(stdflags_cf);i++) {
    if (stdflags_cf[i]==flag) {
      c=1;
    }
  }
  return c;
}

unsigned int isvalidcm(char flag) {
  int i,c;
  for(c=0,i=0;i<strlen(stdflags_cm);i++) {
    if (stdflags_cm[i]==flag) {
      c=1;
    }
  }
  return c;
}

unsigned int findchannel(char *chan) {
  PGresult *res;
  strtolower(chan);
  res=pg_query("SELECT * FROM channels WHERE lower(channel)='%s';",chan);
  if (!res) {
    PQclear(res);
    return 0;
  }
  if (pg_num_rows(res)<1) {
    PQclear(res);
    return 0;
  } else {
    PQclear(res);
    return 1;
  }
}

int getcid(char *chan) {
  PGresult *res; int result_id; chans *a;
  a=cptr(chan);
  if (a==NULL) return -1;
  if (!a->cid || a->cid==0) {
    strtolower(chan);
    res=pg_query("SELECT cid FROM channels WHERE lower(channel)='%s';",chan);
    if (pg_num_rows(res)<1) return -1;
    result_id=atoi(PQgetvalue(res,0,0));
    a->cid=result_id;    
    PQclear(res);
  }
  return a->cid;
}

void getchanbyid(int cid, char *result) {
  PGresult *res; chans *a;
  for(a=chan_list;a!=NULL;a=a->next) {
    if (a->cid && a->cid==cid) {
      resetchar(result);
      strcpy(result,a->name);
      result[strlen(a->name)]=0x00;
      return;
    }
  }
  res=pg_query("SELECT channel FROM channels WHERE cid=%d;",cid);
  if (pg_num_rows(res)<1) {
    PQclear(res);
    result=NULL;
    return;
  }
  resetchar(result);
  strcpy(result,PQgetvalue(res,0,0));
  result[strlen(PQgetvalue(res,0,0))]=0x00;
  PQclear(res);
}
  
int getusercount(char *chanx) {
  cusers *a; int b=0;
  for(a=cuser_list;a!=NULL;a=a->next) {
	if (!a->name) continue;
    if (!strncmp(a->name,chanx,stdc_name)) b++;
  }
  return b;
}

void cf_checkchans() {
  chans *a; int x,x2,chans_checked=0,chans_leaved=0; long mintime;
  x2=atoi(mconfig_get("cf_mincount","4"));
  for(a=chan_list;a!=NULL;a=a->next) {
    x=getusercount(a->name);
    if (x>=x2) {
      cf_checkchan(a);
      chans_checked++;
    } else {
      chans_leaved++;
    }
  }
  if (global_debug && (chans_checked>0||chans_leaved>0)) {
    flog("cf_checkchans","%d channel%s checked, %d channel%s leaved.",chans_checked,(chans_checked==1) ? "" : "s", chans_leaved,(chans_leaved==1) ? "" : "s");
  }
  cf_lastcheck=time(NULL);
  mintime=time(NULL) - atoi(mconfig_get("cf_reopmax","864000"));
  pg_query("DELETE FROM cfdata WHERE lastopped<%ld;",mintime);
}

void cf_checkchan(chans *a) {
  cusers *b; regopu *r,*r2=NULL; users *c; int found=0,i,ops=0; char tmp[250];
  PGresult *res;
  /* first update the join counts */
  if (findchannel(a->name)) {
    resetchar(tmp);
    strcp(tmp,a->name);
    strtolower(tmp);
    pg_query("UPDATE channels SET joincount=joincount+%ld WHERE lower(channel)='%s';",a->joins,tmp);
    a->joins=0;
  }
  for(b=cuser_list;b!=NULL;b=b->next) {
    if (!strcmp(b->name,a->name)) {
      found=0;
      if (IsOpped(b->flags)) {
        c=uptr(b->unum);
        if (c==NULL) continue;
        ops++;
        resetchar(tmp);
        strcp(tmp,a->name);
        strtolower(tmp);
        res=pg_query("SELECT score FROM cfdata WHERE lower(chan)='%s' AND uhash='%s';",tmp,c->hash);
        if (pg_num_rows(res)>0) {
          pg_query("UPDATE cfdata SET score=score+1, lastopped='%ld' WHERE lower(chan)='%s' AND uhash='%s';",time(NULL),tmp,c->hash);
        } else {
          pg_query("INSERT INTO cfdata (chan,uhash,score,lastopped) VALUES ('%s','%s',1,'%ld');",a->name,c->hash,time(NULL));
        }
        PQclear(res);
      }
    }
  }
  if (ops==0) cf_reop(a);
}

regops *cf_createregops(chans *a) {
  regops *c;
  if ((c=(regops *)myalloc(sizeof(regops))) == NULL) {
    flog("cf_createregops","FATAL ERROR: myalloc() returned NULL!");
    return NULL;
  }
  c->chan=a->name;
  c->created=time(NULL);
  c->lastcheck=0;
  c->next=regop_list;
  regop_list=c;
  return c;
}

int cf_getregopcount(char *chan) {
  PGresult *res; int result=0;
  strtolower(chan);
  res=pg_query("SELECT * FROM cfdata WHERE lower(chan)='%s';",chan);
  result=pg_num_rows(res);
  PQclear(res);
  return result;
}
   
int cf_reop(chans *a) {
  /* reopping opless channels
     first, we have to find suitable scores to be opped, if not, leave them alone. */
  PGresult *res; cusers *b; users *c;
  char ulist[520],cmo[10],tmpchan[300]; int users_opped=0;
  resetchar(tmpchan);
  strcp(tmpchan,a->name);
  strtolower(tmpchan);
  res=pg_query("SELECT * FROM cfdata WHERE lower(chan)='%s';",tmpchan);
  if (pg_num_rows(res)<1) {
    /* no scores in database */
    PQclear(res);
    return;
  }
  PQclear(res);
  resetchar(ulist);
  resetchar(cmo);
  for(b=cuser_list;b!=NULL;b=b->next) {
    if (!strcmp(b->name,a->name)) {
      c=uptr(b->unum);
      if (c==NULL) continue;
      resetchar(tmpchan);
      strcp(tmpchan,a->name);
      strtolower(tmpchan);
      res=pg_query("SELECT * FROM cfdata WHERE lower(chan)='%s' AND uhash='%s' AND score>=4;",tmpchan,c->hash);
      if (pg_num_rows(res)>0) {
        /* suitable op, we should op him */
        strcat(ulist,b->unum);
        strcat(ulist," ");
        strcat(cmo,"o");
        if (strlen(cmo)>=stddef_maxmpln) {
          ulist[strlen(ulist)-1]=0x00;
          send2up("%s M +%s %s",mconfig_get("numeric","JP"),cmo,ulist);
          resetchar(ulist);
          resetchar(cmo);
        }
        SetOpped(b->flags);
        users_opped++;
      }
      PQclear(res);
    }
  }
  if (strlen(cmo)>0) {
    ulist[strlen(ulist)-1]=0x00;
    send2up("%s M %s +%s %s",mconfig_get("numeric","JP"),a->name,cmo,ulist);
    resetchar(ulist);
    resetchar(cmo);
  }
  if (mconfig_get_bool("cf_notify","false") && users_opped>0) {
    sprintf(ulist,"Fixed opless channel '%s': %d user%s opped.",a->name,users_opped,(users_opped==1) ? "" : "s");
    m_privmsg(mconfig_get("debugchan","#bluelightzone"),getmodsnum("core"),ulist);
  }
}

regops *regopptr(char *chan) {
  regops *a;
  for(a=regop_list;a!=NULL;a=a->next) {
    if (!strcmp(a->chan,chan)) return a;
  }
  return NULL;
}

void avg_checkchans() {
  chans *a; int uc=0;
  for(a=chan_list;a!=NULL;a=a->next) {
    uc=getusercount(a->name);
    if (uc<=atoi(mconfig_get("avg_mincount","10"))) continue;
    avg_makesnapshot(a,uc);
  }
}

void avg_makesnapshot(chans *chan, int usercount) {
  PGresult *res; float avg=0; int cc=0;
  strtolower(chan->name);
  res=pg_query("SELECT avg,checks FROM avgdata WHERE lower(channel)='%s';",chan->name);
  if (pg_num_rows(res)<1) {
    flog("avg_makesnapshot","num_rows <1: Adding.. (%s)\n",chan->name);
    pg_query("INSERT INTO avgdata (avg,checks,firstcheck,channel) VALUES (%d,1,'%ld','%s');",usercount,time(NULL),chan->name);
    PQclear(res);
    return;
  }
  flog("avg_makesnapshot","Continue.. (%s)\n",chan->name);
  avg=atof(PQgetvalue(res,0,0));
  cc=atoi(PQgetvalue(res,0,1));
  avg+=usercount;
  avg=avg/2;
  cc++;
  PQclear(res);
  pg_query("UPDATE avgdata SET avg=%0.2f, checks=%d WHERE channel~*'%s';",avg,cc,chan->name);
}

void update_autolimits(void) {
  PGresult *res; char *chan; int i,alim; char prfx1[50];
  resetchar(prfx1);
  if (mconfig_get_bool("cs_alserv","false")) {
    sprintf(prfx1,"%s M",mconfig_get("numeric","JP"));
  } else {
    sprintf(prfx1,"%s M",getmodsnum("cservice"));
  }
  res=pg_query("SELECT channel,autolimit FROM channels WHERE flags LIKE '%s' AND autolimit>0;","%c%");
  if (pg_num_rows(res)>0) {
    for(i=0;i<pg_num_rows(res);i++) {
      chan=PQgetvalue(res,i,0);
      alim=atoi(PQgetvalue(res,i,1));
      alim+=getusercount(chan);
      send2up("%s %s +l %d",prfx1,chan,alim);
    }
  }
}

void SetOpped(char *ptr) {
  if (ptr!=NULL && !chrstr('o',ptr)) {
    strcat(ptr,"o");
  }
}

void SetVoice(char *ptr) {
  if (ptr!=NULL && !chrstr('v',ptr)) {
    strcat(ptr,"v");
  }
}

int IsOpped(char *ptr) {
  return chrstr('o',ptr);
}

int IsVoiced(char *ptr) {
  return chrstr('v',ptr);
}

void ClrOpped(char *ptr) {
  if (ptr!=NULL) {
    delchar(ptr,'o');
  }
}

void ClrVoiced(char *ptr) {
  if (ptr!=NULL) {
    delchar(ptr,'v');
  }
}
