/**********************************************************
 * ModServ2x - Modular IRC Services for advanced use.     *
 *   Based on the Undernet/Beware P10 Protocol.           *
 *   (C) 2004-2005 Christian Jurk (commx@commx.ws)        *
 *   Special thanks goes to:                              *
 *     Stephan Peijnik                                    *
 *     Linus Kimselius                                    *
 * ------------------------------------------------------ *
 * This program is released under the terms of the GNU    *
 * Public License (GPL). You'll find the License in the   *
 * docs directory shipped with this release.              *
 * ------------------------------------------------------ *
 * clients.c                                              *
 * Client-based functions.                                *
 * 21/04/05 11:48 commx                                   *
 **********************************************************/

#include "include/main.h"
#include "include/ucdata.h"
#include "include/clients.h"
#include "include/pgsql.h"
#include "include/channels.h"
#include "include/configx.h"
#include "include/yalloc.h"
#include "include/trusts.h"

void client_create(char *mod, char *nick, char *ident, char *host, char *modes, unsigned int auth, char *real) {
  myclient *a,*b; users *c; unsigned int i,j,w=0; char tmpnum[20],tmps[20],tmpauth[512];
  for(a=mycli_list;a!=NULL;a=a->next) {
    assert(a->user->nick != 0);
    if (!strcasecmp(nick,a->user->nick)) {
      printf("*** client_create(): cannot create fakeclient '%s', the nick already exists!\n",nick);
      return;
    }
  }
  strcpy(tmpnum,mconfig_get("numeric","YO"));  
  srand(time(NULL));
  for(;;) {
    w=0;
    for(j=1;j<=3;j++) {
      i=rand()%strlen(modserv_base64);
      sprintf(tmps,"%c",modserv_base64[i]);
      strcat(tmpnum,tmps);
    }
    for(a=mycli_list;a!=NULL;a=a->next) {
      assert(a->user->unum != 0);
      if (!strcmp(tmpnum,a->user->unum)) {
        // numeric exists already, restart the loop
        tmpnum[0]=0x00;
        strcpy(tmpnum,mconfig_get("numeric","YO"));
        w=1; break;
      }
    }
    if (w) { continue; }
    break; 
  }
  /* numeric generated, create the client! */
  c=(users *)myalloc(sizeof(users));
  strcp(c->nick,nick);
  strcp(c->ident,ident);
  strcp(c->host,host);
  strcp(c->modes,modes);
  strcp(c->unum,tmpnum);
  sprintf(tmpauth,"auth_%s",nick);
  getmd5(tmpauth,c->hash);
  c->connected=time(NULL);
  c->using_tg=0;
  c->servcli=mTrue;
  strcp(c->addr,"127.0.0.1");
  if (auth) {
    strcp(c->auth,nick);
  } else {
    c->auth[0]='\0';
  }
  c->next=user_list;
  user_list=c;
  b=(myclient*)myalloc(sizeof(users));
  b->user=c;
  strcp(b->module,mod);
  b->next=mycli_list;
  mycli_list=b;
  send2up("%s N %s 1 %ld %s %s %s%s%s BC]]] %s :%s",mconfig_get("numeric","YO"),nick,time(NULL),ident,host,modes,(auth==1) ? " " : "",(auth==1) ? nick : "",tmpnum,real);
}

void client_close(char *nick, char *quitmsg) {
  myclient *a,*c=NULL; users *b,*d=NULL; char *tmpnum;
  tmpnum=nick2num(nick);
  if (tmpnum!=NULL) {
    send2up("%s Q :%s",tmpnum,quitmsg);
  }
  killuser(tmpnum);
  for(a=mycli_list;a!=NULL;a=a->next) {
    assert(a->user->nick != 0);
    if (!strcasecmp(a->user->nick,nick)) {
      if (c == NULL) {
        mycli_list=a->next;
      } else {
        c->next=a->next;
      }
      free(a);
      break;
    }
    c = a;
  }  
}

void client_join(char *nick, char *chan) {
  users *a; uchans *u; cusers *c; char unum[20];
  for(a=user_list;a!=NULL;a=a->next) {
    assert(a->nick != 0);
    if (!strcasecmp(a->nick,nick)) {
      adduchan(a->unum,chan,time(NULL));
      addcuser(chan,a->unum,time(NULL));
      send2up("%s J %s :%ld",a->unum,chan,time(NULL));
      send2up("%s M %s +o %s",snum,chan,a->unum);
      setcumode(chan,a->unum,cm_o);
      return;
    }
  }
}

void client_part(char *nick, char *chan) {
  users *a; uchans *u;
  for(a=user_list;a!=NULL;a=a->next) {
    assert(a->nick != 0);
    if (!strcasecmp(a->nick,nick)) {
      delcuser(chan,a->unum);
      send2up("%s L %s",a->unum,chan);
      return;
    }
  }
}

void setcumode(char *chan, char *unum, unsigned int modes) {
  cusers *c;
  for(c=cuser_list;c!=NULL;c=c->next) {
    assert(c->name != 0);
    assert(c->unum != 0);
    if (!strcmp(c->name,chan) && !strcmp(c->unum,unum)) {
      if (HasMode(modes,'o')) {
        SetOpped(c->flags);
      }
      if (HasMode(modes,'v')) {
        SetVoice(c->flags);
      }
      break;
    }
  }
}

void getmodnum(char *mod, char *res) {
  myclient *c;
  for(c=mycli_list;c!=NULL;c=c->next) {
    assert(c->module != 0);
    if (!strcmp(c->module,mod)) {
      res[0]=0x00;
      res=c->user->unum;
      return;
    }
  }
}

char *num2nick(char *unum) {
  users *a;
  for(a=user_list;a!=NULL;a=a->next) {
    assert(a->unum != 0);
    if (!strcmp(a->unum,unum)) {
      return a->nick;
    }
  }
  return NULL;
}

char *nick2num(char *nick) {
  users *a;
  for(a=user_list;a!=NULL;a=a->next) {
    assert(a->nick != 0);
    if (!strcasecmp(a->nick,nick)) {
      return a->unum;
    }
  }
  return NULL;
}

int isoper(char *unum) {
  users *a;
  for(a=user_list;a!=NULL;a=a->next) {
    assert(a->unum != 0);
    if (!strcmp(a->unum,unum)) {
      if (chrstr('o',a->modes)) {
        return 1;
      } else {
        return 0;
      }
    }
  }
}

users *uptr(char *unum) {
  users *a;
  for(a=user_list;a!=NULL;a=a->next) {
    assert(a->unum != 0);
    if (!strcmp(a->unum,unum)) {
      return a;
    }
  }
  return NULL;
}

unsigned int isauthed(char *unum) {
  users *a; char *b; PGresult *res;
  // 13-02-2005, commx: Added check for existing accounts.
  if ((b=num2nick(unum))==NULL) return;
  for(a=user_list;a!=NULL;a=a->next) {
    assert(a->unum != 0);
    if (!strcmp(a->unum,unum)) {
      if (chrstr('r',a->modes)) {
        return 1;
      } else {
        return 0;
      }
    }
  }
}  

char *getauth(char *unum) {
  users *a;
  if (!unum || unum[0]=='\0' || unum==NULL) return NULL;
  for(a=user_list;a!=NULL;a=a->next) {
    assert(a->unum != 0);
    if (!strcmp(a->unum,unum)) {
      if (!chrstr('r',a->modes) || a->auth[0]=='\0') {
        return NULL;
      } else {
        return a->auth;
      }
    }
  }
}

void killuser(char *unum) {
  cusers *a,*a2=NULL; uchans *b,*b2=NULL; users *c,*c2=NULL;
  trusthost *th;
  for(a=cuser_list;a!=NULL;a=a->next) {
    assert(a->unum != 0);
    if (!strcmp(a->unum,unum)) {
      if (a2 == NULL) {
        cuser_list=a->next;
      } else {
        a2->next=a->next;
      }
      myfree(a->unum);
      myfree(a->name);
      myfree(a);
      break;
    }
    a2 = a;
  }
  for(b=uchan_list;b!=NULL;b=b->next) {
    assert(b->unum != 0);
    if (!strcmp(b->unum,unum)) {
      if (b2 == NULL) {
        uchan_list=b->next;
      } else {
        b2->next=b->next;
      }
      myfree(b->unum);
      myfree(b->name);
      myfree(b);
      break;
    }
    b2 = b;
  }
  for(c=user_list;c!=NULL;c=c->next) {
    assert(c->unum != 0);
    if (!strcmp(c->unum,unum)) {
      if (c2 == NULL) {
        user_list=c->next;
      } else {
        c2->next=c->next;
      }
      for(th=th_list;th!=NULL;th=th->next) {
        assert(th->addr != 0);
        if (!strcmp(th->addr,c->addr)) {
          th->usage--;
          break;
        }
      }
      myfree(c);
      break;
    }
    c2 = c;
  }
}

char *uflags(char *account) {
  PGresult *res; char *rex,*a;
  strtolower(account);
  res=pg_query("SELECT flags FROM users WHERE lower(account)='%s';",account);
  if (pg_num_rows(res)<1) {
    PQclear(res);
    return NULL;
  }
  a=PQgetvalue(res,0,0);
  if ((rex=(char*)(malloc(strlen(a))))==NULL) {
    slog("** FATAL ERROR: Cannot allocate memory uflags()!");
    return NULL;
  }
  strncpy(rex,a,strlen(a));
  return rex;
}

char *getuflags(char *account) {
  char *b; PGresult *res;
  strtolower(account);
  res=pg_query("SELECT flags FROM users WHERE lower(account)='%s';",account);
  if (pg_num_rows(res)<1) {
    PQclear(res);
    return NULL;
  }
  b=(char*)myalloc(strlen(PQgetvalue(res,0,0))+1);
  strcp(b,PQgetvalue(res,0,0));
  return b;
}

int hasuflag(char *account, char flag) {
  char *a;
  a=getuflags(account);
  if (a==NULL) { return -1; }
  if (chrstr(flag,a)) {
    free(a);
    return 1;
  } else {
    free(a);
    return 0;
  }
}

int adduflag(char *account, char flag) {
  PGresult *res; char *flags;
  strtolower(account);
  res=pg_query("SELECT flags FROM users WHERE lower(account)='%s';",account);
  if (pg_num_rows(res)<1) {
    PQclear(res);
    return 0;
  }
  flags=PQgetvalue(res,0,0);
  appchar(flags,flag);
  strsortn(flags);
  pg_query("UPDATE users SET flags='%s' WHERE lower(account)='%s'",flags,account);
  PQclear(res);
  return 1;
}

int deluflag(char *account, char flag) {
  PGresult *res; char *flags;
  strtolower(account);
  res=pg_query("SELECT flags FROM users WHERE lower(account)='%s'",account);
  if (pg_num_rows(res)<1) {
    PQclear(res);
    return 0;
  }
  flags=PQgetvalue(res,0,0);
  delchar(flags,flag);
  pg_query("UPDATE users SET flags='%s' WHERE lower(account)='%s'",flags,account);
  PQclear(res);
  return 1;
}

int isvaliduflag(char flag) {
  int i,c=0;
  for(i=0;i<strlen(stdflags_uf);i++) {
    if (stdflags_uf[i]==flag) {
      c=1;
    }
  }
  return c;
}

int getuserid(char *name) {
  PGresult *res; int uid;
  strtolower(name);
  res=pg_query("SELECT uid FROM users WHERE lower(account)='%s'",name);
  if (pg_num_rows(res)>0) {
    uid=atoi(PQgetvalue(res,0,0));
    PQclear(res);
    printf("GetUID (%s) (%d)\n",name,uid);
    return uid;
  } else {
    PQclear(res);
    printf("GetUID FAILED! (%s)\n",name);
    return -1;
  }
}

char *gu(char *item, int tyy) {
  PGresult *res; users *a; char tmpauth[40];
  switch(tyy) {
    case 1:
      /* Numeric to Account-Lang */
      a=uptr(item);
      resx[0]=0x00;
      if (a==NULL) {
        strcpy(resx,"english");
        resx[7]=0x00;
        return resx;
      } else {
        if (a->lang[0]!='\0')
          return a->lang;
      }
      if (!isauthed(item)) {
        strcpy(resx,"english");
        resx[7]=0x00;
      } else {
        resetchar(tmpauth);
        strcp(tmpauth,getauth(item));
        strtolower(tmpauth);
        res=pg_query("SELECT lang FROM users WHERE lower(account)='%s' AND lang!='' AND lang!='english'",tmpauth);
        if (pg_num_rows(res)<1) {
          strcpy(resx,"english");
          resx[7]=0x00;
        } else {
          strcpy(resx,PQgetvalue(res,0,0));
          resx[strlen(PQgetvalue(res,0,0))]=0x00;
        }
        if (a->lang[0]=='\0') {
          strcpy(a->lang,resx);
          a->lang[strlen(resx)]=0x00;
        }
        PQclear(res);
      }
  }
  return resx;
}
