/**********************************************************
 * ModServ2x - Modular IRC Services for advanced use.     *
 *   Based on the Undernet/Beware P10 Protocol.           *
 *   (C) 2004-2005 Christian Jurk (commx@commx.ws)        *
 *   Special thanks goes to:                              *
 *     Stephan Peijnik                                    *
 *     Linus Kimselius                                    *
 * ------------------------------------------------------ *
 * This program is released under the terms of the GNU    *
 * Public License (GPL). You'll find the License in the   *
 * docs directory shipped with this release.              *
 * ------------------------------------------------------ *
 * config.c                                               *
 * Config file reader.                                    *
 * 21/04/05 11:51 commx                                   *
 **********************************************************/
   
#include "include/main.h"
#include "include/configx.h"
#include "include/yalloc.h"
#include "include/memstr.h"

void mconfig_add(char *t, char *cf, char *a, char *v) {
  mconfig *c;
  c=(mconfig *)myalloc(sizeof(mconfig));
  strcp(c->item,a);
  strcp(c->val,v);
  strcp(c->cat,cf);
  if (!strcasecmp(t,"B")) {
    c->type=TC_BOOL;
  } else if (!strcasecmp(t,"S")) {
    c->type=TC_STRING;
  } else if (!strcasecmp(t,"I")) {
    c->type=TC_INTEGER;
  } else {
    c->type=TC_UNKNOWN;
  }
  c->len=strlen(v);
  c->next=config_list;
  config_list=c;
}

void mconfig_del(char *item) {
  mconfig *c,*d=NULL;
  for(c=config_list;c!=NULL;c=c->next) {
    assert(c->item);
    if (!strcmp(item,c->item)) {
      if (d==NULL) {
        config_list=c->next;
      } else {
        d->next=c->next;
      }
      free(c);
    }
    d=c;
  }
}

char *mconfig_get(char *item, char *def) {
  mconfig *c;
  for(c=config_list;c!=NULL;c=c->next) {
    assert(c->item);
    if (!strcmp(c->item,item)) {
      c->val[c->len]=0x00;
      return c->val;
    }
  }
  return def;
}

int mconfig_get_bool(char *item, char *def) {
  mconfig *c; int res=0;
  for(c=config_list;c!=NULL;c=c->next) {
    assert(c->item);
    if (!strcmp(c->item,item)) {
      c->val[c->len]=0x00;
      if (!strcmp(c->val,"true")) {
        res=1;
      } else if (!strcmp(c->val,"false")) {
        res=0;
      } else {
        if (!strcmp(def,"true")) {
          res=1;
        } else {
          res=0;
        }
      }
    }
  }
  return res;
}

int mconfig_get_integer(char *item, int def) {
  mconfig *c; int res,found=0;
  for(c=config_list;c!=NULL;c=c->next) {
    assert(c->item);
    if (!strcmp(c->item,item)) {
      c->val[c->len]=0x00;
      res=atoi(c->val);
      found=1;
    }
  }
  if (!found) {
    res=def;
  }
  return res;
}

void delchar(char *a, char b) {
  char *c, *d;
  c=a; d=a;
  while (*c!='\0') {
    if (*c!=b) {
      *d=*c;
      c++; d++;
    } else {
      c++;
    }
  }
  *d='\0';
}

unsigned int mconfig_read(char *file) {
  mconfig *c; FILE *cf; char *res; char tm[1024],t1[10],t2[256],t3[256],t4[256]; unsigned int l=0;
  for(c=config_list;c!=NULL;c=c->next) {
    myfree(c);
  }
  cf=fopen(file,"r");
  if (cf!=NULL) {
    for(;(res=fgets(tm,1000,cf))!=NULL;) {
      resetchar(t1);
      resetchar(t2);
      resetchar(t3);
      resetchar(t4);
      sscanf(tm,"%[^:]%[^/]%[^=]%[^\n]",t1,t2,t3,t4);
      delchar(t2,':');
      delchar(t3,'/');
      delchar(t4,'=');
      if (t1[0]=='B' || t1[0]=='S' || t1[0]=='I') {
        mconfig_add(t1,t2,t3,t4);
        l++;
      }
    }
  }
  return l;
}
