/**********************************************************
 * ModServ2x - Modular IRC Services for advanced use.     *
 *   Based on the Undernet/Beware P10 Protocol.           *
 *   (C) 2004-2005 Christian Jurk (commx@commx.ws)        *
 *   Special thanks goes to:                              *
 *     Stephan Peijnik                                    *
 *     Linus Kimselius                                    *
 * ------------------------------------------------------ *
 * This program is released under the terms of the GNU    *
 * Public License (GPL). You'll find the License in the   *
 * docs directory shipped with this release.              *
 * ------------------------------------------------------ *
 * dynhdl.c                                               *
 * Dynamic handlers for commands/modules.                 *
 * 21/04/05 11:52 commx                                   *
 **********************************************************/
   
#include "include/main.h"
#include "include/dynhdl.h"
#include "include/ucdata.h"

void shl_add(char *mod, char *tok, void *func) {
  shl *a;
  a=(shl *)myalloc(sizeof(shl));
  a->token=memstr_new(tok);
  a->module=memstr_new(mod);
  a->func=func;
  a->next=shl_list;
  shl_list=a;
}

void shl_del(char *mod, char *tok) {
  shl *a,*b=NULL;
  for(a=shl_list;a!=NULL;a=(void*)a->next) {
    if (!strcmp(a->module->str,mod) && !strcmp(a->token->str,tok)) {
      if (b == NULL) {
        shl_list=a->next;
      } else {
        b->next = a->next;
      }
      memstr_free(a->token);
      memstr_free(a->module);
      free(a);
      break;
    }
    b = a;
  }
}

void shl_run(char *tok) {
  shl *a; int i=0;
  for(a=shl_list;a!=NULL;a=a->next,i++) {
    if (!strcmp(a->token->str,tok)) {
      a->func();
    }
  }
}

void chl_add(char *mod, char *cmd, void *func, int o, int t, int auth, unsigned int desc) {
  chl *a;
  a=(chl *)myalloc(sizeof(chl));
  strcp(a->command,cmd);
  strcp(a->module,mod);
  a->desc=desc;
  a->func=func;
  a->type=t;
  a->oper=o;
  a->auth=auth;
  a->active=1;
  a->next=cmd_list;
  cmd_list=a;
}

void chl_del(char *mod, char *cmd, int t) {
  chl *a,*b=NULL;
  for(a=cmd_list;a!=NULL;a=a->next) {
    if (!strcmp(a->module,mod) && !strcmp(a->command,cmd) && a->type==t) {
      if (b == NULL) {
        cmd_list=a->next;
      } else {
        b->next=a->next;
      }
      free(a);
      break;
    }
    b = a;
  }
}

void chl_run(char *cmd, char *unum, char *target, char *arg, int t) {
  chl *a;
  for(a=cmd_list;a!=NULL;a=a->next) {
    if (!strcmp(a->command,cmd) && a->type==t && a->active) {
      if (a->auth && !isauthed(unum)) {
        m_notice(unum,target,"You must be authed to use this command.");
        return;
      }
      a->func(arg,target,unum);
    }
  }
}      

int loadmod(char *name, char *moddesc) {
  void *load; void(*ix)(void); mhl *mod; char newmod[2048],initcmd[256]; char *mp;
  FILE *cf; char *res; char tm[1024],t1[128],t2[128],t3[128]; unsigned int l=0; int i;
  mp=(char*)mconfig_get("modpath","./modules/");
  if (global_debug) {
    printf("** loadmod(): Module '%s'\n",name);
  }
  sprintf(newmod,"%s%s%s",mp,name,mconfig_get("modsuffix",".so"));
  load=dlopen(newmod,RTLD_NOW);
  if (load==NULL) {
    printf("** Module error: %s\n",dlerror());
    return 0;
  }
  mod=(mhl *)myalloc(sizeof(mhl));
  mod->name=memstr_new(name);
  mod->desc=memstr_new(moddesc);
  mod->loadtime=time(NULL);
  mod->func=load;
  mod->next=mod_list;
  mod_list=mod;
  initcmd[0]=0x00;
  sprintf(initcmd,"%s_init",name);
  ix=dlsym(load,initcmd);
  if (ix==NULL) {
    slog("** Module error: %s_init() failed, but module was loaded.",name);
    return 1;
  }
  ix();
  // loading module commands
  initcmd[0]=0x00;
  sprintf(initcmd,"%s%s/%s.commands",mp,name,name);
  cf=fopen(initcmd,"r");
  if (cf!=NULL) {
    for(;(res=fgets(tm,1000,cf))!=NULL;) {
      if (tm[0]!='\0' && tm[0]!='#' && tm[0]!='/') {
        sscanf(tm,"%s %s %s",t1,t2,t3);
        t3[strlen(t3)]=0x00;
        if (strlen(t3)<1) { continue; }
        delchar(t1,'\n');
        i=loadcommand(name,t1,t2,t3);
        if (i==1) { l++; }
        t1[0]=0x00; t2[0]=0x00; t3[0]=0x00;
      }
    }
    fclose(cf);
  }
  if (l>0) { slog("** Module '%s': %d %s loaded.",name,l,(l>1) ? "commands" : "command"); }
  return 1;
}

int loadcommand(char *module, char *oper, char *type, char *cmd) {
  void *load; void (*sym)(void); char initcmd[256]; chl *c;
  if (global_debug) {
    printf("** loadcommand(): Module '%s' (%s)\n",module,cmd);
  }
  if (atoi(type)==1) {
    sprintf(initcmd,"%s%s/%s%s",mconfig_get("modpath","./modules/"),module,cmd,mconfig_get("modsuffix",".so"));
  } else {
    sprintf(initcmd,"%s%s/pub/%s%s",mconfig_get("modpath","./modules/"),module,cmd,mconfig_get("modsuffix",".so"));
  }
  load=dlopen(initcmd,RTLD_NOW);
  if (load==NULL) {
    slog("** Command error: %s",dlerror());
    return 0;
  }
  sprintf(initcmd,"load_%s_%s_%s",module,(atoi(type)==1) ? "cmd" : "pub",cmd);
  sym=dlsym(load,initcmd);
  if (sym==NULL) {
    slog("** Command error: load_%s_%s_%s failed, but command was loaded.\n",module,(atoi(type)==1) ? "cmd" : "pub",cmd);
    return 1;
  }
  sym();
  for(c=cmd_list;c!=NULL;c=c->next) {
    if (!strcmp(c->module,module) && !strcmp(c->command,cmd) && atoi(type) == c->type) {
      c->dlptr = load;
      break;
    }
  }
  return 1;
}

int unloadcmd(char *module, char *command, int type) {
  chl *c;
  for(c=cmd_list;c!=NULL;c=c->next) {
    if (!strcmp(c->command,command) && !strcmp(c->module,module) && c->type == type) {
      if (c->dlptr!=NULL) {
        dlclose(c->dlptr);
        chl_del(module,command,type);
        return 1;
      }
    }
  }
  return 0;
}

int modstate(char *module) {
  mhl *m;
  for(m=mod_list;m!=NULL;m=m->next) {
    if (!strcmp(m->name->str,module)) {
      return 1;
    }
  }
  return 0;
}

int unloadmod(char *module) {
  void(*ix)(void); mhl *m; chl *c; char closecmd[256];
  for(m=mod_list;m!=NULL;m=m->next) {
    if (!strcmp(m->name->str,module)) {
      closecmd[0]=0x00; sprintf(closecmd,"%s_close",module);
      ix=dlsym(m->func,closecmd);
      if (ix!=NULL) { ix(); }
      for(c=cmd_list;c!=NULL;c=c->next) {
        if (!strcmp(c->module,module)) {
          dlclose(c->dlptr);
          chl_del(c->module,c->command,c->type);
          break;
        }
      }
      dlclose(m->func);
      return 1;
    }
  }
  return 0;
}

int setdesc(char *module, char *desc) {
  mhl *m;
  for(m=mod_list;m!=NULL;m=m->next) {
    if (!strcmp(m->name->str,module)) {
      memstr_replace(m->desc,desc);
      return 1;
    }
  }
  return 0;
}

int module_autoload() {
  FILE *cf; char *res; char tm[1024],t1[128],t2[256],t3[512]; unsigned int l=0; int i;
  cf=fopen(modfile,"r");
  if (cf!=NULL) {
    for(;(res=fgets(tm,1000,cf))!=NULL;) {
      delchar(tm,'\n');
      sscanf(tm,"%[^=]%[^:]%[^\n]",t1,t2,t3);
      delchar(t2,'=');
      delchar(t3,':');				
      if (tm[0]!='\0' && tm[0]!='#' && tm[0]!='/') {
        i=loadmod(t2,t3);
        if (i==1) { l++; }
      }
    }
    fclose(cf);
  }
  return l;
}

char *getmodnick(char *mod) {
  myclient *a;
  for(a=mycli_list;a!=NULL;a=a->next) {
    if (!strncmp(a->module,mod,40)) {
      return a->user->nick;
    }
  }
  return NULL;
}

char *getmodsnum(char *mod) {
  myclient *a;
  for(a=mycli_list;a!=NULL;a=a->next) {
    if (!strncmp(a->module,mod,40)) {
      return a->user->unum;
    }
  }
  return NULL;
}

int cmdstate(char *cmd, char *mod) {
  chl *a;
  for(a=cmd_list;a!=NULL;a=a->next) {
    if ((!strcmp(a->module,mod)) && (!strcmp(a->command,cmd))) {
      if (a->active) {
        return 1;
      } else {
        return 2;
      }
    }
  }
  return 0;
}
