/**********************************************************
 * ModServ2x - Modular IRC Services for advanced use.     *
 *   Based on the Undernet/Beware P10 Protocol.           *
 *   (C) 2004-2005 Christian Jurk (commx@commx.ws)        *
 *   Special thanks goes to:                              *
 *     Stephan Peijnik                                    *
 *     Linus Kimselius                                    *
 * ------------------------------------------------------ *
 * This program is released under the terms of the GNU    *
 * Public License (GPL). You'll find the License in the   *
 * docs directory shipped with this release.              *
 * ------------------------------------------------------ *
 * lang.c                                                 *
 * Multi-language functions.                              *
 * 21/04/05 11:53 commx                                   *
 **********************************************************/

#include "include/main.h"
#include "include/lang.h"
#include "include/strfunc.h"
#include "include/configx.h"

unsigned int loadlang(char *datei, char *lang) {
  FILE *cf; char *res; char tm[1024],t1[128],t2[1024]; unsigned int l=0; int i; langlist *x;
  cf=fopen(datei,"r");
  if (cf!=NULL) {
    for(;(res=fgets(tm,1000,cf))!=NULL;) {
      delchar(tm,'\n');
      sscanf(tm,"%[^=]%[^\n]",t1,t2);
      delchar(t2,'=');
      if (tm[0]!='\0' && tm[0]!='#' && tm[0]!='/' && tm[0]!='[') {
        i=addlangitem(atoi(t1),t2,lang);
        if (i==1) { l++; }
      }
    }
  }
  if (l>0) {
    slog("** Loading language '%s' (%s): %d %s loaded.",lang,datei,l,(l==1) ? "item" : "items");
  }
  x=(langlist *)malloc(sizeof(langlist));
  strncpy(x->name,lang,strlen(lang));
  x->name[strlen(lang)]=0x00;
  x->next=lang_list;
  lang_list=x;  
  return l;
}

unsigned int addlangitem(int i, char *v, char *l) {
  dlang *e; int j,k; mBool mailx=mFalse;
  e=(dlang *)malloc(sizeof(dlang));
  e->item=i;
  for(j=0;j<strlen(v);j++) {
    if (v[j]=='*') {
      v[j]='\02';
      j=0; continue;
    } else if (v[j]=='_') {
      v[j]='\03';
      j=0; continue;
    } else if (v[j]=='|' && v[j+1]=='|') {
      v[j]='\r';
      v[j+1]='\n';
      mailx=mTrue;
      j++;
    }
  }
  if (mailx == mFalse) {
    delchar(v,'\n');
    delchar(v,'\r');
  }
  strcp(e->val,v);
  strcp(e->lang,l);
  e->next=dlang_list;
  dlang_list=e;
  return 1;
}

int dellang(char *lang) {
  dlang *e,*b=NULL; langlist *f,*g=NULL; int i=0;
  for(e=dlang_list;e!=NULL;e=e->next) {
    if (!strcmp(e->lang,lang)) {
      if (b==NULL) {
        dlang_list=e->next;
      } else {
        b->next=e->next;
      }
      i=1;
      free(e);
      break;
    }
    b = e;
  }
  for(f=lang_list;f!=NULL;f=f->next) {
    if (!strcmp(f->name,lang)) {
      if (g==NULL) { lang_list=f->next; } else { g->next=f->next; }
      i=1;
      free(f);
    }
    g = f;
  }
  return i;
}

void autoload_lang(void) {
  FILE *cf; char *res,*res2; char tm[1024],t1[128],t2[1024]; unsigned int l=0; int i;
  sprintf(t2,"conf/%s",mconfig_get("langfile","modserv.msl"));
  if (!(cf=fopen(t2,"r"))) {
    slog("Cannot open '%s': autoload_lang()!",t2);
    return;
  }
  res2=malloc(5);
  strcpy(res2,mconfig_get("langext",".msu")); i=0;
  if (cf!=NULL) {
    for(;(res=fgets(tm,1000,cf))!=NULL;) {
      delchar(tm,'\n'); t2[0]=0x00;
      sscanf(tm,"%[^\n]",t1);
      if (tm[0]!='\0' && tm[0]!='#' && tm[0]!='/') {
        sprintf(t2,"%s%s%s",mconfig_get("langdir","./lang/"),t1,res2);
        i=loadlang(t2,t1);
        if (i>0) l++;
      }
    }
  }
  if (global_debug>0 && l>0) {
    slog(">> Autoloading languages: %d %s loaded.",l,(l>1) ? "languages" : "language");
  }
}

char *getlang(int item, char *language) {
  dlang *a;
  for(a=dlang_list;a!=NULL;a=a->next) {
    if (a->item==item && !strcmp(a->lang,language)) {
      return a->val;
    }
  }
  return NULL;
}
