/**********************************************************
 * ModServ2x - Modular IRC Services for advanced use.     *
 *   Based on the Undernet/Beware P10 Protocol.           *
 *   (C) 2004-2005 Christian Jurk (commx@commx.ws)        *
 *   Special thanks goes to:                              *
 *     Stephan Peijnik                                    *
 *     Linus Kimselius                                    *
 * ------------------------------------------------------ *
 * This program is released under the terms of the GNU    *
 * Public License (GPL). You'll find the License in the   *
 * docs directory shipped with this release.              *
 * ------------------------------------------------------ *
 * modserv2x.c                                            *
 * The core of ms2x.                                      *
 * 21/04/05 11:56 commx                                   *
 **********************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include "include/main.h"
#include "include/socket.h"
#include "include/dynhdl.h"
#include "include/ucdata.h"
#include "include/lang.h"
#include "include/schedule.h"
#include "include/configx.h"
#include "include/servers.h"
#include "include/channels.h"
#include "include/strfunc.h"
#include "include/account.h"
#include "include/trusts.h"
#include "include/yalloc.h"
#include "include/log.h"

int main(int argn, char *argv[]) {
  unsigned int lc,lm,rv=0,hl=0,li,i=1,c=0,tp,tl=1; char buf[530],line[530],*tok; mBool create_db;
  struct pollfd pfd;
  startup=time(NULL);
  fillnull();
  global_debug=0; 
  resetchar(tmpv1);
  resetchar(tmpv2);
  have_eob=0;
  bytecounter=0;
  trusts_loaded = mFalse;
  dolog         = mFalse;
  load_modules  = mTrue;
  create_db     = mFalse;
  if (!strcmp(modserv_ircdvar,"kotaro")) {
    received_vctrl=1;
  }
  for(i=0;i<argn;i++) {
    if (!strncmp(argv[i],"--debug",7) || !strcmp(argv[i],"-d")) {
      global_debug=atoi(gettok(argv[i],"=",2));
      slog("* Global debug enabled, level: %d",global_debug);
      continue;
    }
    if (!strncmp(argv[i],"--config",8) || !strcmp(argv[i],"-c")) {
      tok=gettok(argv[i],"=",2);
      c=1; continue;
    }
    if (!strcmp(argv[i],"--version") || !strcmp(argv[i],"-v")) {
      printf("ModServ %s\n",modserv_version); exit(1);
    }
    if (!strcmp(argv[i],"--no-trusts") || !strcmp(argv[i],"-t")) {
      tl=0;
      continue;
    }
    if (!strcmp(argv[i],"--no-automod") || !strcmp(argv[i],"-m")) {
      load_modules = mFalse;
      continue;
    }
    if (!strcmp(argv[i],"--create-dbstruct") || !strcmp(argv[i],"-s")) {
      create_db=mTrue;
      continue;
    }
    if (!strncmp(argv[i],"--logging",9) || !strcmp(argv[i],"-l")) {
      dolog=mTrue;
      if (strlen(argv[i])>10) {
        enable_logging(gettok(argv[i],"=",2));
      } else {
        enable_logging(NULL);
      }
      continue;
    }
    if (!strcmp(argv[i],"--help") || !strcmp(argv[i],"-h")) {
      printf("Usage: %s [-cdiv] [ARGS]...\n",argv[0]);
      printf("Starts the ModServ irc service with the given parameters.\n\n");
      printf("Program arguments:\n");
      printf("  -c, --config  [=FILE]   Starts with a given config file.\n");
      printf("  -d, --debug   [=LEVEL]  Activates debugging output.\n");
      printf("  -h, --help              Displays this Program info.\n");
      printf("  -l, --logging [=FILE]   Enables logging in the given file.\n");
      printf("  -m, --no-automod        Disables auto-loading of modules.\n");
      printf("  -s, --create-dbstruct   Clears a database and writes all new tables to it.\n");
      printf("  -t, --no-trusts         Disables loading of trust data.\n");
      printf("  -v, --version           Returns the version.\n");
      exit(1);
    }
  }
  printf(">> ModServ %s starting.\n",modserv_version);
  if (c>0) {
    printf("** Read config (%s)... ",tok);
    lc=mconfig_read(tok);
  } else {
    printf("** Read config (%s)... ",configfile);
    lc=mconfig_read(configfile);
  }
  if (lc==0) { printf("failed, exiting.\n"); exit(-1); }
  snum=mconfig_get("numeric","YO");
  printf("succeed, %d %s loaded.\n",lc,(lc>1) ? "settings" : "setting");
  printf("** Connecting to database... ");
  pg_connect();
  printf("succeed!\n");
  if (create_db == mTrue) {
    create_database();
  }  
  printf(">> Connecting to %s, port %d... ",mconfig_get("host","localhost"),atoi(mconfig_get("port","4400")));
  sfd=core_sock_connect(mconfig_get("host","localhost"),atoi(mconfig_get("port","4400")));
  if (sfd<0) { printf("failed, exiting.\n"); exit(-1); }
  printf("succeed, identify...\n");
  send2up("PASS :%s",mconfig_get("pass","password"));
  send2up("SERVER %s 1 %ld %ld J10 %s]]] +s :ModServ %s",mconfig_get("servername","modserv.default.org"),time(NULL),time(NULL),mconfig_get("numeric","YO"),modserv_version);
  addserver(mconfig_get("servername","modserv.default.org"),mconfig_get("numeric","JP"),time(NULL),time(NULL),"+s",mconfig_get("numeric","JP"));
  if (load_modules == mTrue) {
    lm=module_autoload();
    printf("** Autostarting modules... ");
    if (lm==0) { printf("failed, exiting.\n"); exit(-1); }
    printf("%d module(s) loaded.\n",lm);
  }
  autoload_lang();
  if (tl) {
    slog("** Loading trusts... ");
    loadtrusts();
  }
  send2up("%s EB",snum);  
  burstlines=0;
  //buf=(char*)malloc(modserv_bufsize*sizeof(char));
  //line=(char*)malloc(modserv_bufsize*sizeof(char)+1);
  /* adding main tasks */
  addtask("core","Chanfix channel scanner",1800,cf_checkchans,TASK_PERM);
  addtask("core","Channel average scanner",1800,avg_checkchans,TASK_PERM);
  /* preparing poll() */
  pfd.fd=sfd;
  pfd.events=POLLIN|POLLERR;
  pfd.revents=0; li=0;
  for(;;) {
    poll(&pfd,1,1); 
    runtasks();
    if (pfd.revents & POLLIN) {
      buf[0]='\0'; line[0]='\0';
      for(;buf[0]!='\n';) {
        i=read(sfd,buf,1);
        if (i<=0) break;
        strncat(line,buf,1);
      }
      line[strlen(line)-1]='\0';
      if (global_debug>=4) {
        slog("[>>]: %s\n",line);
      }
      core_get_argv(line);
      if (!have_eob) {
        burstlines++;
      }
      shl_run(args[1]);
      if (!strcmp(args[0],"SERVER")) { 
        shl_lserver();
        continue;
      }
      if (!strcmp(args[0],"ERROR")) {
        /* received some error message from target server */
        slog("Received ERROR message from target server:");
        slog(line);
        break;
      }
    } else if ((pfd.revents & POLLHUP) || (pfd.revents & POLLERR)) {
      break;
    }
  }
  myprintf("*** Connection to %s:%s closed.\n",mconfig_get("host","localhost"),mconfig_get("port","4400"));
  cleanup();
  return 0;
}

void core_get_argv(char *line) {
  int i=0,c=0; char *a;
  argc=0; a=&line[0];
  for(;((*a!='\n') && (*a!='\0') && (*a!='\r') && (argc<28));a++) {
    if ((c==0) && (*a==' ')) {
      args[argc][i]=0x00;
      argc++; i=0;
      while(*(a+1)==' ') { c++; }
    } else {
      if ((c==0) && (*a==':') && (i==0) && (argc>1)) {
        c=1;
      } else {
        args[argc][i]=*a;
        i++;
      }
    }
  }
  args[argc][i]=0x00;
  argc++;
}

void send2up(char *str, ...) {
  va_list ap; char ts[1024];
  va_start(ap,str);
  vsnprintf(ts,modserv_bufsize,str,ap);
  strcat(ts,"\r\n");
  send(sfd,ts,strlen(ts),0);
  if (global_debug>=3) {
    slog("[<<]: %s",ts);
  }
  va_end(ap);
}

void shl_lserver() {
  char *corenum;
  hubnum=(char*)malloc(3);
  strncpy(hubnum,args[6],2);
  hubnum[2]=0x00;
  addserver(args[1],hubnum,atol(args[3]),atol(args[4]),"+h",hubnum);
  client_create("core",mconfig_get("corenick","M2"),mconfig_get("coreident","control"),mconfig_get("corehost","modserv.default.org"),"+idkr",1,mconfig_get("corereal","ModServ 2.0 ControlStation"));
  client_join(mconfig_get("corenick","M2"),mconfig_get("staffchan","#bluelightzone"));
}

void fillnull() {
  user_list=NULL;
  chan_list=NULL;
  cuser_list=NULL;
  uchan_list=NULL;
  mycli_list=NULL;
  acc_list=NULL;
  mod_list=NULL;
  cmd_list=NULL;
  shl_list=NULL;
  lang_list=NULL;
  dlang_list=NULL;
  sched_list=NULL;
  serv_list=NULL;
  regop_list=NULL;
  regopu_list=NULL;
  tg_list=NULL;
  th_list=NULL;
  config_list=NULL;
}     

void cleanup(void) {
  chl *a; mhl *b; shl *c; unsigned int i=0;
  for(a=cmd_list;a!=NULL;a=a->next) {
    cmd_list=a->next;
    free(a->command);
    free(a->module);
    free(a); i++;
  }
  for(b=mod_list;b!=NULL;b=b->next) {
    mod_list=b->next;
    memstr_free(b->name);
    memstr_free(b->desc);
    free(b); i++;
  }
  for(c=shl_list;c!=NULL;c=c->next) {
    shl_list=c->next;
    memstr_free(c->token);
    memstr_free(c->module);
    free(c); i++;
  }
  slog("** cleanup(): %d items free'd.",i);
}
