/* ...............................................................
   : ModServ, modular IRC services, version 2.                   :
   : core.c, core module                                         :
   : (c) 2004 Christian Jurk, portions Stephan Peijnik (dstring) :
   : --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- :
   : This programm is free software, you can redistribute and/or :
   : modify it under the terms of the GNU Public License (GPL).  :
   : You should have received a copy of the license along with   :
   : this program. If not, write to the Free Software Foundation :
   : Inc.,59 Temple Place - Suite 330,Boston,MA 02111-1307, USA. :
   :.............................................................: */
   
#include "main.h"
#include "ucdata.h"
#include "channels.h"
#include "dynhdl.h"
#include "configx.h"
#include "strfunc.h"
#include "modbas.h"
#include "clients.h"
#include "socket.h"
#include "trusts.h"
#include "schedule.h"

#define module_name "core"
#define module_desc "Main functions/serverhandlers"

/* A      AWAY            [not added]
 * AC     ACCOUNT         [not added]
 * B      BURST           core_shl_burst()
 * C      CREATE          core_shl_create()
 * CM     CLEARMODE       core_shl_clearmode()
 * EB     END_OF_BURST    core_shl_eob()
 * D      KILL            core_shl_kill()
 * G      PING            core_shl_ping()
 * I      INVITE          [not added]
 * J      JOIN            core_shl_join()
 * K      KICK            [not added]
 * L      PART            core_shl_part()
 * M      MODE            core_shl_mode()
 * N      NICK            core_shl_nick()
 * O      NOTICE          [not added]
 * OM     OPMODE          core_shl_mode()
 * P      PRIVMSG         core_shl_privmsg()
 * Q      QUIT            core_shl_quit()
 * S      SERVER          core_shl_server()
 * SQ     SQUIT           core_shl_serverquit()
 * T      TOPIC           core_shl_topic()
 * VC     VCTRL           core_shl_vctrl()
 */
 
/* core_shl_burst */
char lastchan[stdc_name],tmpcf[40];

void core_shl_ping(void) {
  send2up("%s Z :%s",mconfig_get("numeric","YO"),args[3]);
}

void core_shl_nick(void) {
  users *a; unsigned int i,r=0; char hashtmp[512]; char *iptr;
  struct hostent *mya; trustgroup *tg; trusthost *th;
  if (argc>5) {
    // new user introduced
    a=(users *)myalloc(sizeof(users));
    strcp(a->nick,args[2]);
    strcp(a->ident,args[5]);
    strcp(a->host,args[6]);
    a->connected = atol(args[4]);
    a->using_tg = 0;
    a->auth[0]='\0';
    if (args[7][0]=='+') {
      // user has modes
      strcp(a->modes,args[7]);
      for(i=0;i<strlen(a->modes);i++) {
        if (a->modes[i]=='r') {
          // user is authed, add auth
          strcp(a->auth,args[8]);
          break;
        }
      }
    } else {
      // user has no modes
      strcpy(a->modes,"+");
      a->modes[1]=0x00;
    }
    strcp(a->unum,args[argc-2]);
    if (a->auth[0]!='\0') {
      sprintf(hashtmp,"auth_%s",a->auth);
    } else {
      sprintf(hashtmp,"mask_%s@%s",a->ident,a->host);
    }
    getmd5(hashtmp,a->hash); /* generate md5 hash */
    a->lang[0]='\0';
    a->next   = user_list;
    user_list = a;
    if ((mya=gethostbyname(a->host))==NULL) {
      /* host not found, probably fake-hosts */
      strcp(a->addr,"0.0.0.0");
    } else {
      assert(mya);
      if ((iptr=inet_ntoa(*(struct in_addr*)mya->h_addr_list[0]))!=NULL) {
        strcp(a->addr,iptr);
      } else {
        strcp(a->addr,"127.0.0.1");
      }
    }
    /* check if the host is trusted */
    if (!mconfig_get_bool("clonescan","true")) {
      return;
    }
    if ((tg=istrusted(a->addr,a))==NULL) {
      if (getucbyhost(a->addr) > mconfig_get_integer("maxclones",5)) {
        //send2up("%s D %s :%s (Too many connections from your host.)",mconfig_get("numeric","JP"),a->unum,mconfig_get("servername","modserv.default.org"));
        //killuser(a->unum);
        printf("Wanna kill (%s/%s) - (%d) > (%d)\n",a->unum,a->addr,getucbyhost(a->addr),mconfig_get_integer("maxclones",5));
        return;
      }
    } else {
      if (getcctg(tg) > tg->connections) {
        send2up("%s D %s :%s (All connections of trustgroup #%d are being used. Please try again later or request a trustgroup-extension.)",mconfig_get("numeric","JP"),a->unum,mconfig_get("servername","modserv.default.org"),tg->id);
        killuser(a->unum);
        return;
      }
      for(th=th_list;th!=NULL;th=th->next) {
        if (!strcmp(a->addr,th->addr)) {
          th->usage++;
          th->lastused=(time_t)time(NULL);
          if (th->usage>th->maxusage) {
            th->maxusage=th->usage;
          }
        }
      }
    }
  } else {
    // nick change detected
    for(a=user_list;a!=NULL;a=a->next) {
      if (!strcmp(a->unum,args[0])) {
         strcp(a->nick,args[2]);
         break;
      }
    }
  }  
}

void core_shl_privmsg(void) {
  myclient *a,*d; chl *b,*e; unsigned int i; char ucmd[128]; char *c; char cpr;
  ucmd[0]=0x00;
  sscanf(args[3],"%s",ucmd);
  if (!strcmp(ucmd,"\01VERSION\01")) {
    m_notice(args[0],args[2],"\01ModServ %s\01",modserv_version);
    m_notice(args[0],args[2],"\01%s %s %s\01",__DATE__,__TIME__,getlang(8201,gu(args[0],1)));
    m_notice(args[0],args[2],"\01Copyright(c) 2004-2005 GamerIRC developement team.\01");
    return;
  }
  if (chrstr('@',args[2])) {
    // nick@server message
    c=(char*)strtok(args[2],"@");
    for(d=mycli_list;d!=NULL;d=d->next) {
      if (!strcasecmp(d->user->nick,c)) {
        for(e=cmd_list;e!=NULL;e=e->next) {
          if (!strcmp(e->module,d->module) && !strcasecmp(e->command,ucmd) && e->active) {
            if (e->oper && !isoper(args[0])) {
              m_notice(args[0],args[2],getlang(8202,gu(args[0],1)));
              return;
            } else if (e->auth && !isauthed(args[0])) {
              m_notice(args[0],args[2],getlang(8203,gu(args[0],1)));
              return;
            }
            e->func(args[3],args[0],d->user->unum);
            return;
          }
        }
      }
    }
    m_notice(args[0],args[2],getlang(8218,gu(args[0],1)),ucmd);
    return;    
  }
  if (args[2][0]=='#') {
    // channel target, public command
    cpr=mconfig_get("pubprfx","!")[0];
    delchar(ucmd,cpr);
    for(b=cmd_list;b!=NULL;b=b->next) {
      if (!strcasecmp(b->command,ucmd) && b->active && b->type==2) {
        if (b->oper && !isoper(args[0])) {
          m_notice(args[0],args[2],getlang(8202,gu(args[0],1)));
          return;
        } else if (b->auth && !isauthed(args[0])) {
          m_notice(args[0],args[2],getlang(8203,gu(args[0],1)));
          return;
        }
        b->func(args[3],args[0],args[2]);
        return;
      }
    }    
  } else {
    // direct target, private command    
    for(a=mycli_list;a!=NULL;a=a->next) {
      if (!strcmp(a->user->unum,args[2])) {
        for(b=cmd_list;b!=NULL;b=b->next) {     
          if (!strcmp(b->module,a->module) && !strcasecmp(b->command,ucmd) && b->active && b->type==1) {
            if (b->oper && !isoper(args[0])) {
              m_notice(args[0],args[2],getlang(8202,gu(args[0],1)));
              return;
            } else if (b->auth && !isauthed(args[0])) {
              m_notice(args[0],args[2],getlang(8203,gu(args[0],1)));
              return;
            }
            b->func(args[3],args[0],args[2]);
            return;
          }
        }
      }
    }
    m_notice(args[0],args[2],getlang(8218,gu(args[0],1)),ucmd);
    return;
  }
}

void core_shl_eob(void) {
  if (!have_eob) {
    send2up("%s EA",mconfig_get("numeric","YO"));
    have_eob=1; linktime=time(NULL);
  }
}

void core_shl_quit(void) {
  killuser(args[0]);
}

void core_shl_kill(void) {
  killuser(args[2]);
}

void core_shl_burst(void) {
  chans *a; unsigned int b=5,i; char *c; char tmpv3[1024],tmpv4[1024];
  cusers *d; uchans *e;
  a=cptr(args[2]);
  if (a==NULL) {
    a=(chans *)myalloc(sizeof(chans));
    strcp(a->name,args[2]);
    a->created=atol(args[3]);
    a->next=chan_list;
    resetchar(a->key);
    a->limit=0;
	a->topic[0]='\0';
    a->regops=NULL;
    a->cid=0;
    a->joins=1;	  
	a->locked=0;
    chan_list=a;
    if (args[4][0]=='+') {
      // channel has modes
      if (chrstr('k',args[4])) {
        strcp(a->key,args[b]);
        b++;
      }
      if (chrstr('l',args[4])) {
        a->limit=atoi(args[b]);
        b++;
      }
      setmodes(a,args[4]);
    } else {
      setmodes(a,NULL);
      b--;
    }
  }
  if (strcmp(lastchan,args[2])) {
    tmpcf[0]='\0';
  }
  tmpv3[0]='\0'; tmpv4[0]='\0';
  c=&args[b][0];
  for(;*c!='\0';c++) {
    i=0;
    for(;((*c!='\0') && (*c!=','));c++) {
      tmpv3[i]=*c;
      i++;
    }
    tmpv3[i]='\0';
    if (strlen(tmpv3)<5) {
      slog("** Warning: Servernumeric in BURST message found!");
      continue;
    }
    if (tmpv3[3]==':') {
      strcpy(tmpv4,&tmpv3[4]);
      tmpv3[3]='\0';
    } else {
      if (tmpv3[5]==':') {
        strcpy(tmpv4,&tmpv3[6]);
        tmpv3[5]='\0';
      } else {
        tmpv4[0]='\0';
      }
    }
    if (tmpv4[0]!='\0') {
      strcp(tmpcf,tmpv4);
    }
    // adding user to channel: tmpv3 is the numeric, tmpcf the flags
    addcuser(args[2],tmpv3,time(NULL));
    adduchan(tmpv3,args[2],time(NULL));
    d=cuptr(args[2],tmpv3);
    if (d!=NULL) {
      d->flags[0]='\0';
      if (chrstr('o',tmpcf)) {
        SetOpped(d->flags);
      }
      if (chrstr('v',tmpcf)) {
        SetVoice(d->flags);
      }
    }
    if (*c=='\0') break;
  }
  lastchan[0]='\0';
  strcp(lastchan,args[2]);
}

void core_shl_create(void) {
  chans *a; cusers *b; uchans *c; char *d; unsigned int done=0;
  if (strlen(args[0])<5) {
    slog("** Warning: A %s (%s) tried to create %s!",(strlen(args[0])==2) ? "server" : "ghost",args[0],args[2]);
    return;
  }
  if (chrstr(',',args[2])) {
    // more channels are created
    d=(char*)strtok(args[2],",");
    newchannel(d,args[0],atol(args[3]));
    for(;d!=NULL;d=(char*)strtok(NULL,",")) {
      newchannel(d,args[0],atol(args[3]));
    }
  } else {
    newchannel(args[2],args[0],atol(args[3]));
  }
}

void core_shl_join(void) {
  char *c; uchans *a;  
  if (!strcmp(args[2],"0")) {
    /* 0 - leave all channels */
    for(a=uchan_list;a!=NULL;a=a->next) {
      delcuser(a->name,a->unum);
    }
    return;
  }
  if (chrstr(',',args[2])) {
    c=(char*)strtok(args[2],",");
    addcuser(c,args[0],atol(args[3]));
    for(;c!=NULL;c=(char*)strtok(NULL,",")) {
      addcuser(c,args[0],atol(args[3]));
    }
  } else {
    addcuser(args[2],args[0],atol(args[3]));
  }
}

void core_shl_part(void) {
  char *c;
  if (chrstr(',',args[2])) {
    c=(char*)strtok(args[2],",");
    delcuser(c,args[0]);
    for(;c!=NULL;c=(char*)strtok(NULL,",")) {
      delcuser(c,args[0]);
    }
  } else {
    delcuser(args[2],args[0]);
  }
}

void core_shl_mode(void) {
  unsigned int i,d=0,b=4; chans *c; cusers *e; users *u;
  if (args[2][0]=='#') {
    // channel mode is changed
    c=cptr(args[2]);
    if (c==NULL) {
      slog("** Warning: Cannot change channel-modes (%s): Unknown channel.",args[2]);
      return;
    }
    for(i=0;i<strlen(args[3]);i++) {
      if (args[3][i]=='+') { 
        d=1;
        continue;
      }
      if (args[3][i]=='-') { 
        d=2;
        continue;
      }
      if (d==1) {
        if (args[3][i]=='o') {
          e=cuptr(args[2],args[b]);
          if (e==NULL) {
            slog("** Warning: Cannot get cuptr! (%s/%s)",args[2],args[b]);
            b++; continue;
          }
          SetOpped(e->flags);
          b++; continue;
        } else if (args[3][i]=='v') {
          e=cuptr(args[2],args[b]);
          if (e==NULL) {
            slog("** Warning: Cannot get cuptr! (%s/%s)",args[2],args[b]);
            b++; continue;
          }
          SetVoice(e->flags);
          b++; continue;
        } else {
          setmode(c,args[3][i]);
        }
        if (args[3][i]=='k') {
          strncpy(c->key,args[b],strlen(args[b]));
          b++; continue;
        } else if (args[3][i]=='l') {
            if (atol(args[b])>300000) {
              slog("** Warning: Limit on channel %s is bigger then 300000! Have NOT set the limit!",c->name);
              b++; continue;
            }
            c->limit=atoi(args[b]);
            b++; continue;
        }
      } else if (d==2) {
        if (args[3][i]=='o') {
          e=cuptr(args[2],args[b]);
          if (e==NULL) {
            slog("** Warning: Cannot get cuptr! (%s/%s)",args[2],args[b]);
            b++; continue;
          }
          ClrOpped(e->flags);
          b++; continue;
        } else if (args[3][i]=='v') {
          e=cuptr(args[2],args[b]);
          if (e==NULL) {
            slog("** Warning: Cannot get cuptr! (%s/%s)",args[2],args[b]);
            b++; continue;
          }
          ClrVoiced(e->flags);
          b++; continue;
        } else {
          delmode(c,args[3][i]);
          if (args[3][i]=='k') {
            dropcc(c->key,0);
            b++; continue;
          } else if (args[3][i]=='l') {
            c->limit=0;
            b++; continue;
          }
        }
      }
    }
  } else {
    // user mode is changed
    /* since users can only change their own modes, we use the sender */
    u=uptr(args[0]);
    if (u==NULL) {
      slog("** Warning: Cannot find uptr() for [%s] (%s)",args[0],args[2]);
      return;
    }
    for(i=0;i<strlen(args[3]);i++) {
      if (args[3][i]=='+') { d=1; continue; }
      if (args[3][i]=='-') { d=2; continue; }
      if (d==1) {
        // adding modes
        appchar(u->modes,args[3][i]);
        continue;
      } else if (d==2) {
        // revoking modes
        delchar(u->modes,args[3][i]);
        continue;
      }
    }
  }
}

void core_shl_kick(void) {
  cusers *a;
  for(a=cuser_list;a!=NULL;a=a->next) {
    if (!strcasecmp(a->name,args[2]) && !strcmp(a->unum,args[3])) {
      cuser_list=a->next;
      free(a->name);
      free(a->unum);
      free(a);
    }
  }
}

void core_shl_server(void) {
  char flags[20];
  // AB S some.server.net 1 1020343274 1927373243 J10 AC]]] +s :desc
  if (args[8][0]=='+') {
    strcpy(flags,args[8]);
  } else {
    strcpy(flags,"+l");
  }
  addserver(args[2],args[7],((time_t)atol(args[4])),((time_t)atol(args[5])),flags,args[0]);
}

void core_shl_serverquit(void) {
  delserver(args[2]);
}

void core_shl_clearmode(void) {
  /* Usage: AC CM #channel :ovb */
  chans *c; cusers *a; int i;
  c=cptr(args[2]);
  if (c==NULL) {
    if (global_debug) {
      flog("core_shl_clearmode","Unable to get cptr (%s)",args[2]);
    }
    return;
  }
  for(i=0;i<strlen(args[3]);i++) {
    if (args[3][i]=='o' || args[3][i]=='v') {
      /* clear cuser modes */
      for(a=cuser_list;a!=NULL;a=a->next) {
        if (!strcmp(a->name,args[2])) {
          if (args[3][i]=='o' && IsOpped(a->flags)) {
            ClrOpped(a->flags);
          } else if (args[3][i]=='v' && IsVoiced(a->flags)) {
            ClrVoiced(a->flags);
          }
        }
      }
    } else {
      /* clear channel modes */
      delchar(c->modes,args[3][i]);
    }
  }
  /* sorting the modes */
  strsortn(c->modes);
}

void core_shl_vctrl(void) {
  /* <nicklen>/<userlen>/<hostlen>/<accountlen>/<reallen>/<passwdlen>/<sockiplen>/<topiclen>/<awaylen> */
  int i;
  for(i=2;i<argc;i++) {
    lengths[i-2]=atoi(args[i]);
  }
  if (!strcmp(modserv_ircdvar,"kotaro")) {
    received_vctrl=1;
  }
  slog("** Setting NICKLEN to %d, TOPICLEN to %d.",lengths[0],lengths[7]);
}

void core_shl_topic(void) {
  chans *a; int i=3; char topic[stdc_topic];
  a=cptr(args[2]);
  if (a==NULL) {
    return;
  }
  resetchar(topic);
  strcp(topic,args[argc-1]);
  if (!strcmp(modserv_ircdvar,"kotaro") && received_vctrl) {
    if (strlen(topic)>lengths[7]) {
      slog("Warning: Topic of %s exceeds TOPICLEN, not saved.",args[2]);
      return;
    }
  } else {
    if (strlen(topic)>stdc_topic) {
      slog("Warning: Topic of %s exceeds TOPICLEN, not saved.",args[2]);
      return;
    }
  }
  /* to make sure the topic is set on all servers, send it out again if we're in burst. */
  if (mconfig_get_bool("synctopics","true") && !have_eob) {
    send2up("%s T %s %ld %ld :%s",mconfig_get("numeric","JP"),args[2],time(NULL),time(NULL),topic);
  }
  /* if cservice protected the topic, don't save the new one */
  if (a->locked==1) {
	a->locked=0;
	return;
  }
  strcp(a->topic,topic);
}

void core_init(void) {
  shl_add(module_name,"B",core_shl_burst);
  shl_add(module_name,"C",core_shl_create);
  shl_add(module_name,"D",core_shl_kill);
  shl_add(module_name,"G",core_shl_ping);
  shl_add(module_name,"J",core_shl_join);
  shl_add(module_name,"K",core_shl_kick);
  shl_add(module_name,"L",core_shl_part);
  shl_add(module_name,"M",core_shl_mode);
  shl_add(module_name,"N",core_shl_nick);
  shl_add(module_name,"P",core_shl_privmsg);
  shl_add(module_name,"Q",core_shl_quit);
  shl_add(module_name,"S",core_shl_server);
  shl_add(module_name,"T",core_shl_topic);
  shl_add(module_name,"CM",core_shl_clearmode);
  shl_add(module_name,"EB",core_shl_eob);
  shl_add(module_name,"OM",core_shl_mode);
  shl_add(module_name,"SQ",core_shl_serverquit);
  shl_add(module_name,"VC",core_shl_vctrl);
  //addtask(module_name,"Saves/Updates trust data.",300,savetrusts,TASK_PERM);
}

void core_close(void) {
  shl_del(module_name,"B");
  shl_del(module_name,"C");
  shl_del(module_name,"D");
  shl_del(module_name,"G");
  shl_del(module_name,"J");
  shl_del(module_name,"K");
  shl_del(module_name,"L");
  shl_del(module_name,"M");
  shl_del(module_name,"N");
  shl_del(module_name,"P");
  shl_del(module_name,"Q");
  shl_del(module_name,"S");
  shl_del(module_name,"T");
  shl_del(module_name,"CM");
  shl_del(module_name,"EB");
  shl_del(module_name,"OM");
  shl_del(module_name,"SQ");
  shl_del(module_name,"VC");
}
