/* ...............................................................
   : ModServ, modular IRC services, version 2.                   :
   : auth.c                                                      :
   : (c) 2004 GamerIRC dev-workx.                                :
   : --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- :
   : This programm is free software, you can redistribute and/or :
   : modify it under the terms of the GNU Public License (GPL).  :
   : You should have received a copy of the license along with   :
   : this program. If not, write to the Free Software Foundation :
   : Inc.,59 Temple Place - Suite 330,Boston,MA 02111-1307, USA. :
   :.............................................................: */

#include "main.h"
#include "modbas.h"
#include "pgsql.h"
#include "ucdata.h"
#include "clients.h"
#include "configx.h"

#define command_module "cservice"
#define command_name   "auth"
#define command_desc   2021
#define command_oper   0
#define command_type   1
#define command_auth   0

#define min_passlen    6

void check_memo_auth(int uid, char *unum, char *target) {
  PGresult *res;
  res=pg_query("SELECT source,sent,message FROM memos WHERE target=%d AND read=FALSE;",uid);
  if (pg_num_rows(res)>0) {
    m_notice(unum,target,getlang(2022,gu(unum,1)),pg_num_rows(res),getmodnick(command_module));
  }
  PQclear(res);
}

void cservice_cmd_auth(char *arg, char *unum, char *target) {
  strarray(arg); PGresult *res,*res2; char tmpvar[256],hashpw[50]; users *a; long countdown; char *ptr; char tmphash[512];
  if (cargc==0) {
    m_notice(unum,target,"\02%s\02 %s",command_name,getlang(command_desc,gu(unum,1)));
    m_notice(unum,target,getlang(2023,gu(unum,1)),command_name);
    m_notice(unum,target,"---");
    m_notice(unum,target,getlang(2024,gu(unum,1)));
    m_notice(unum,target,getlang(2025,gu(unum,1)));
    return;
  } else if (cargc<2) {
    m_notice(unum,target,getlang(8200,gu(unum,1)),getmodnick(command_module),command_name);
    return;
  }
  strtolower(cargs[0]);
  res=pg_query("SELECT account,passwd,lastauth,activated,created,uid FROM users WHERE lower(account)='%s';",cargs[0]);
  if (PQntuples(res)<1) {
    m_notice(unum,target,getlang(2026,gu(unum,1)),cargs[0]);
    PQclear(res);
    return;
  }
  /* hash the password */
  resetchar(hashpw);
  getmd5(cargs[1],hashpw);
  if (strcmp(PQgetvalue(res,0,1),hashpw)) {
    m_notice(unum,target,getlang(2027,gu(unum,1)));
    pg_query("UPDATE users SET authfails=authfails+1 WHERE lower(account)='%s';",cargs[0]);
    PQclear(res);
    return;
  }
  if (!strcmp(PQgetvalue(res,0,3),"f")) {
    countdown=atol(PQgetvalue(res,0,4))+172800;
    if ((long)time(NULL)>countdown) {
      pg_query("DELETE FROM users WHERE lower(account)='%s';",cargs[0]);
      m_notice(unum,target,getlang(2026,gu(unum,1)),cargs[0]);
      PQclear(res);
      return;
    }
    m_notice(unum,target,getlang(2028,gu(unum,1)));
    tmpvar[0]=0x00;
    durationcp(countdown-(long)time(NULL),tmpvar);
    m_notice(unum,target,getlang(2029,gu(unum,1)),cctime(countdown),tmpvar);
    PQclear(res);
    return;
  }
  res2=pg_query("SELECT reason,expirets FROM accountsuspends WHERE uid=%d;",PQgetvalue(res,0,5));
  if (pg_num_rows(res2)>0) {
    m_notice(unum,target,getlang(2030,gu(unum,1)));
    m_notice(unum,target,getlang(2031,gu(unum,1)),PQgetvalue(res2,0,0));
    m_notice(unum,target,getlang(2032,gu(unum,1)),cctime((time_t)atol(PQgetvalue(res2,0,1))));
    PQclear(res);
    PQclear(res2);
    return;
  }
  PQclear(res2);
  a=uptr(unum);
  if (a==NULL) {
    PQclear(res);
    return;
  }
  if (chrstr('r',a->modes)) {
    m_notice(unum,target,getlang(2348,gu(unum,1)));
    PQclear(res);
    return;
  }
  strcp(a->auth,PQgetvalue(res,0,0));
  strcat(a->modes,"r");
  strsortn(a->modes); /* sort the modes now */
  resetchar(tmphash);
  sprintf(tmphash,"auth_%s",a->auth);
  getmd5(tmphash,a->hash);
  m_notice(unum,target,getlang(2033,gu(unum,1)),PQgetvalue(res,0,0));
  if (!strcmp(PQgetvalue(res,0,2),"0")) {
    m_notice(unum,target,getlang(2034,gu(unum,1)),mconfig_get("network","GamerIRC"));
  }
  check_memo_auth(atoi(PQgetvalue(res,0,5)),unum,target);
  pg_query("UPDATE users SET lastauth='%ld',authcount=authcount+1 WHERE lower(account)='%s';",time(NULL),cargs[0]);
  send2up("%s AC %s :%s",snum,unum,PQgetvalue(res,0,0));
  res2=pg_query("SELECT vhost FROM vhosts WHERE uid=%ld AND vdefault=TRUE;",PQgetvalue(res,0,5));
  if (pg_num_rows(res2)>0) {
    assert(PQgetvalue(res2,0,0));
    send2up("%s SH %s :%s",getmodsnum(command_module),num2nick(unum),PQgetvalue(res2,0,0));
    m_notice(unum,target,"\02auto-vhost\02 %s",PQgetvalue(res2,0,0));
    pg_query("UPDATE vhosts SET usec=usec+1 WHERE uid=%ld;",PQgetvalue(res,0,5));
  }
  PQclear(res);
  PQclear(res2);
}

void load_cservice_cmd_auth(void) {
  chl_add(command_module,command_name,cservice_cmd_auth,command_oper,command_type,command_auth,command_desc);
}
