/**********************************************************
 * ModServ2x - Modular IRC Services for advanced use.     *
 *   Based on the Undernet/Beware P10 Protocol.           *
 *   (C) 2004-2005 Christian Jurk (commx@commx.ws)        *
 *   Special thanks goes to:                              *
 *     Stephan Peijnik                                    *
 *     Linus Kimselius                                    *
 * ------------------------------------------------------ *
 * This program is released under the terms of the GNU    *
 * Public License (GPL). You'll find the License in the   *
 * docs directory shipped with this release.              *
 * ------------------------------------------------------ *
 * pgsql.c                                                *
 * PostgreSQL database functions.                         *
 * 21/04/05 12:00 commx                                   *
 **********************************************************/

#include "include/pgsql.h"
#include "include/memstr.h"
#include "include/configx.h"
#include "include/main.h"
#include "include/log.h"

unsigned int pg_connect() {
  char costr[1024];
  sprintf(costr,"host=%s port=%d dbname=%s user=%s password=%s",mconfig_get("pghost","localhost"),mconfig_get_integer("pgport",5432),mconfig_get("pgdb","modserv2"),mconfig_get("pguser","null"),mconfig_get("pgpass","md5"));
  pgc = PQconnectdb(costr);
  //pgc=PQsetdb(mconfig_get("pghost","localhost"),mconfig_get("pgport","5432"),NULL,NULL,mconfig_get("pgdb","modserv2"));
  if (PQstatus(pgc) != CONNECTION_OK) {
    printf("failed: \n** %s\n",PQerrorMessage(pgc));
    PQfinish(pgc);
    exit(-1);
  }
  pg_startup=time(NULL);
  pg_querys=0;
  return 1;
}

PGresult *pg_query(char *str, ...) {
  va_list ap; char cp[8200]; PGresult *pgr;
  va_start(ap,str);
  vsnprintf(cp,2048,str,ap);
  pgr=PQexec(pgc,cp);
  if (!pgr) {
    if (global_debug) slog("** Database query failed: \n** %s",cp);
    return pgr;
  }
  if (global_debug) {
    printf("DEBUG: (Query #%ld): %s\n",pg_querys,cp);
  }
  pg_querys++;
  va_end(ap);  
  return pgr;
}

void pg_close(PGconn *c) {
  PQfinish(c);
}

int pg_num_rows(PGresult *res) {
  return PQntuples(res);
}

void create_database(void) {
  if (pg_connect()<1) {
    slog("create_database: failed; could not establish database-connection.");
    return;
  }
  slog("** creating table 'accountsuspends'...");
  pg_query("CREATE TABLE accountsuspends \
            (\
              reason varchar(250) NOT NULL DEFAULT '<none>'::character varying,\
              setby varchar(25) NOT NULL,\
              setat varchar(10) NOT NULL,\
              expirets varchar(10) NOT NULL,\
              uid int8 NOT NULL\
            ) WITHOUT OIDS;");
  
  slog("** creating table 'avgdata'...");
  pg_query("CREATE TABLE avgdata \
            (\
              firstcheck varchar(10) NOT NULL,\
              checks int4 NOT NULL DEFAULT 0,\
              channel varchar(200) NOT NULL,\
              avg float4\
            ) WITHOUT OIDS;");

  slog("** creating table 'bans'...");
  pg_query("CREATE TABLE bans \
            (\
			  hostmask varchar(255) NOT NULL,\
			  expirets varchar(10) NOT NULL,\
			  banby varchar(25) NOT NULL,\
			  banat varchar(10) NOT NULL,\
			  breason varchar(302) NOT NULL DEFAULT 'no reason given.'::character varying,\
			  cid int8 NOT NULL\
			) WITHOUT OIDS;");
			
  slog("** creating table 'cfdata'...");
  pg_query("CREATE TABLE cfdata \
            (\
			  uhash varchar(32) NOT NULL,\
			  score int4 NOT NULL DEFAULT 0,\
			  lastopped varchar(10) NOT NULL,\
			  chan varchar(200) NOT NULL\
			) WITHOUT OIDS;");

  slog("** creating table 'chanlevs'...");
  pg_query("CREATE TABLE chanlevs \
            (\
			  account varchar(20) NOT NULL,\
			  flags varchar(20) NOT NULL DEFAULT '+'::character varying,\
			  cid int8 NOT NULL\
			) WITHOUT OIDS;");
			
  slog("** creating table 'channels'...");
  pg_query("CREATE TABLE channels \
            (\
              channel varchar(202) NOT NULL,\
              regtime varchar(10) NOT NULL,\
              suspended bool NOT NULL DEFAULT false,\
              joincount int8 NOT NULL DEFAULT 0,\
              cid int8 NOT NULL DEFAULT nextval('inc_cid'::text),\
              flags varchar(15) NOT NULL DEFAULT '+'::character varying,\
			  autolimit int4 NOT NULL DEFAULT 0\
            ) WITHOUT OIDS;");
			
  slog("** creating table 'glines'...");
  pg_query("CREATE TABLE glines \
            (\
              mask varchar(200) NOT NULL,\
              addedby varchar(20) NOT NULL,\
              addedat varchar(10) NOT NULL,\
              expirets varchar(10) NOT NULL,\
              reason varchar(250) NOT NULL DEFAULT 'no reason given'::character varying\
            ) WITHOUT OIDS;");
			
  slog("** creating table 'greetings'...");
  pg_query("CREATE TABLE greetings \
            (\
              message varchar(300) NOT NULL,\
              setby varchar(25) NOT NULL,\
              setat varchar(10) NOT NULL,\
              cid int8 NOT NULL\
            ) WITHOUT OIDS;");
			
  slog("** creating table 'memos'...");
  pg_query("CREATE TABLE memos \
            (\
              source int8 NOT NULL,\
              target int8 NOT NULL,\
              message varchar(260) NOT NULL,\
              mtype int2 NOT NULL,\
              sent varchar(10) NOT NULL,\
              haveread bool NOT NULL DEFAULT false\
            ) WITHOUT OIDS;");
			
  slog("** creating table 'ochannels'...");
  pg_query("CREATE TABLE ochannels \
            (\
              channel varchar(200) NOT NULL\
            ) WITHOUT OIDS;");

  slog("** creating table 'rops'...");
  pg_query("CREATE TABLE rops \
            (\
              channel varchar(200) NOT NULL,\
              nick varchar(20) NOT NULL,\
              ts varchar(10) NOT NULL,\
              result int2,\
              opped int4 NOT NULL DEFAULT 0\
            ) WITHOUT OIDS;");
			
  slog("** creating table 'topics'...");
  pg_query("CREATE TABLE topics \
            (\
              topic varchar(300) NOT NULL,\
              addedby varchar(25) NOT NULL,\
              addedat varchar(10) NOT NULL,\
              cid int8 NOT NULL\
            ) WITHOUT OIDS;");
			
  slog("** creating table 'trustadmins'...");
  pg_query("CREATE TABLE trustadmins \
            (\
              trustgroup int4 NOT NULL DEFAULT 0,\
              account varchar(20) NOT NULL,\
              \"level\" varchar(10) NOT NULL DEFAULT 'co-admin'::character varying\
            ) WITHOUT OIDS;");
			
  slog("** creating table 'trustgroups'...");
  pg_query("CREATE TABLE trustgroups \
            (\
              name varchar(60) NOT NULL,\
              contact varchar(300) NOT NULL,\
              \"comment\" varchar(300) NOT NULL DEFAULT '[no comment]'::character varying,\
              addedby varchar(20) NOT NULL,\
              addedat varchar(10) NOT NULL DEFAULT '0'::character varying,\
              maxperident int4 NOT NULL DEFAULT 1,\
              connections int4 NOT NULL DEFAULT 0,\
              forceident bool NOT NULL DEFAULT true,\
              expires varchar NOT NULL DEFAULT '0'::character varying,\
              id int4 NOT NULL DEFAULT 0\
            ) WITH OIDS;");
			
  slog("** creating table 'trusthosts'...");
  pg_query("CREATE TABLE trusthosts \
            (\
              address varchar(16) NOT NULL DEFAULT '127.0.0.1'::character varying,\
              maxusage int4 NOT NULL DEFAULT 0,\
              lastused varchar(10) DEFAULT '0'::character varying,\
              trustgroup int4 NOT NULL DEFAULT 0\
            ) WITH OIDS;");
			
  slog("** creating table 'userlevels'...");
  pg_query("CREATE TABLE userlevels \
            (\
              vtype int4 NOT NULL DEFAULT 0,\
              vdesc varchar(300) NOT NULL,\
              vid int4 NOT NULL DEFAULT nextval('inc_ulv'::text)\
            ) WITHOUT OIDS;");

  slog("** creating table 'users'...");
  pg_query("CREATE TABLE users \
            (\
              account varchar(20) NOT NULL,\
              passwd varchar(40) NOT NULL,\
              email varchar(255) NOT NULL,\
              lastauth varchar(10) NOT NULL DEFAULT '0'::character varying,\
              created varchar(10) NOT NULL DEFAULT '0'::character varying,\
              authcount int4 NOT NULL DEFAULT 0,\
              authfails int4 NOT NULL DEFAULT 0,\
              suspended bool NOT NULL DEFAULT false,\
              ulevel varchar(80) NOT NULL DEFAULT 'user'::character varying,\
              uid varchar(8) NOT NULL DEFAULT nextval('inc_uid'::text),\
              activated bool NOT NULL DEFAULT false,\
              actcode varchar(20) NOT NULL,\
              flags varchar(20) NOT NULL DEFAULT '+'::character varying,\
              lang varchar(20) DEFAULT 'english'::character varying,\
              realname varchar(60) NOT NULL DEFAULT '<none>'::character varying,\
              loc varchar(40) NOT NULL DEFAULT '<none>'::character varying,\
              bday varchar(10) NOT NULL DEFAULT '00-00-0000'::character varying,\
              lastlogin varchar(10) NOT NULL DEFAULT '0'::character varying\
            ) WITHOUT OIDS;");
			
  slog("** creating table 'vhosts'...");
  pg_query("CREATE TABLE vhosts \
            (\
              uid int8 NOT NULL,\
              vhost varchar(70) NOT NULL,\
              usec int4 NOT NULL DEFAULT 0,\
              vdefault bool NOT NULL DEFAULT false\
            ) WITHOUT OIDS;");

  slog("** creating sequences...");
  pg_query("CREATE SEQUENCE inc_cid INCREMENT 1 MINVALUE 1 MAXVALUE 9223372036854775807 START 1 CACHE 1;");
  pg_query("CREATE SEQUENCE inc_uid INCREMENT 1 MINVALUE 1 MAXVALUE 9223372036854775807 START 1 CACHE 1;");
  pg_query("CREATE SEQUENCE inc_ulv INCREMENT 1 MINVALUE 1 MAXVALUE 9223372036854775807 START 1 CACHE 1;");

  slog("create_database: all tables have been created.");
}
