/**********************************************************
 * ModServ2x - Modular IRC Services for advanced use.     *
 *   Based on the Undernet/Beware P10 Protocol.           *
 *   (C) 2004-2005 Christian Jurk (commx@commx.ws)        *
 *   Special thanks goes to:                              *
 *     Stephan Peijnik                                    *
 *     Linus Kimselius                                    *
 * ------------------------------------------------------ *
 * This program is released under the terms of the GNU    *
 * Public License (GPL). You'll find the License in the   *
 * docs directory shipped with this release.              *
 * ------------------------------------------------------ *
 * schedule.c                                             *
 * Task manager system.                                   *
 * 21/04/05 12:01 commx                                   *
 **********************************************************/

#include "include/main.h"
#include "include/schedule.h"

int global_task_id=0;

unsigned int addtask(char *mod, char *desc, unsigned int ival, void *func, int type) {
  schedule *c;
  for(c=sched_list;c!=NULL;c=c->next) {
    if (!strcmp(mod,c->module) && !strcmp(desc,c->desc)) {
      if (c->interval!=ival) {
        c->interval=ival;
      } else {
        slog("** Error: addtask(): Task '%s' for module '%s' already exists!\n",desc,mod);
        return;
      }
    }
  }
  c=(schedule *)myalloc(sizeof(schedule));
  c->module=(char*)(malloc(strlen(mod)));
  strncpy(c->module,mod,strlen(mod)); c->module[strlen(mod)]=0x00;
  c->desc=(char*)(malloc(strlen(desc)));
  strncpy(c->desc,desc,strlen(desc)); c->desc[strlen(desc)]=0x00;
  c->interval=ival;
  c->func=func;
  c->type=type;
  c->taskid=gettaskid();
  c->runs=0;
  c->lastact=time(NULL);
  c->next=sched_list;
  sched_list=c;
  if (global_debug) {
    slog("** Adding new task '%s' (%s), interval %d.\n",desc,mod,ival);
  }
  return c->taskid;
}

void runtasks(void) {
  schedule *c;
  for(c=sched_list;c!=NULL;c=c->next) {
    if ((c->lastact+c->interval)<time(NULL)) {
      c->func();
      c->lastact=time(NULL);
      if (c->runs<100000) {
        c->runs++;
      }
      if (global_debug) {
        slog("** Launching task '%s' [%ld] [%ld] [interval %d]\n",c->desc,time(NULL),c->lastact,c->interval);
      }
      if (c->type==TASK_ONCE) {
        deltask(c->taskid);
      }
    }
  }
}

void deltask(unsigned int taskid) {
  schedule *c,*c2=NULL; unsigned int i=0;
  for(c=sched_list;c!=NULL;c=c->next) {
    if (c->taskid==taskid) {
      if (c2==NULL) {
        sched_list=c->next;
      } else {
        c2->next=c->next;
      }
      free(c->module);
      free(c->desc);
      free(c); i=1;
    }
    c2=c;
  }
  if ((global_debug) && (i)) {
    slog("** Removed task: '%d'.\n",taskid);
  }
}

void deltasks(char *modname) {
  schedule *c,*c2=NULL; unsigned int i=0;
  for(c=sched_list;c!=NULL;c=c->next) {
    if (!strcmp(modname,c->module)) {
      if (c2==NULL) {
        sched_list=c->next;
      } else {
        c2->next=c->next;
      }
      free(c->module);
      free(c->desc);
      free(c); i=1;
	  c=sched_list;
	  continue;
    }
    c2=c;
  }
}

unsigned int gettaskid() {
  global_task_id++;
  return global_task_id;
}
