/**********************************************************
 * ModServ2x - Modular IRC Services for advanced use.     *
 *   Based on the Undernet/Beware P10 Protocol.           *
 *   (C) 2004-2005 Christian Jurk (commx@commx.ws)        *
 *   Special thanks goes to:                              *
 *     Stephan Peijnik                                    *
 *     Linus Kimselius                                    *
 * ------------------------------------------------------ *
 * This program is released under the terms of the GNU    *
 * Public License (GPL). You'll find the License in the   *
 * docs directory shipped with this release.              *
 * ------------------------------------------------------ *
 * strfunc.c                                              *
 * String manipulation functions.                         *
 * 21/04/05 12:04 commx                                   *
 **********************************************************/

#include "include/main.h"
#include "include/strfunc.h"
#include "include/md5.h"

void strtoupper(char *a) {
  unsigned int i;
  for(i=0;i<strlen(a);i++) {
    a[i]=toupper(a[i]);
  }
}

void strtolower(char *a) {
  unsigned int i;
  for(i=0;i<strlen(a);i++) {
    a[i]=tolower(a[i]);
  }
}

char *inttostr(int i, char *res) {
  res[0]=0x00;
  sprintf(res,"%d",i);
}

unsigned int chrstr(unsigned int c, char *s) {
  unsigned int i;
  for(i=0;i<strlen(s);i++) {
    if (s[i]==c) { return 1; }
  }
  return 0;
}

void mailsend(char *target, char *subject, char *str, ...) {
  va_list ap; char mail[4096],pipe[1024]; FILE *smtp;
  sprintf(pipe,"/usr/sbin/sendmail %s",target);
  va_start(ap,str);
  vsprintf(mail,str,ap);
  smtp=popen(pipe,"w");
  if (smtp==NULL) {
    slog("** mailsend(): cannot send email to '%s' (%s)",target,subject);
    return;
  }
  fprintf(smtp,"From: %s <services@%s>\r\n",mconfig_get("network","GamerIRC"),mconfig_get("servername","modserv.default.org"));
  fprintf(smtp,"To: %s\r\n",target);
  fprintf(smtp,"Subject: %s\r\n",subject);
  fprintf(smtp,mail);
  pclose(smtp);
  va_end(ap);
}

void dropcc(char *c, unsigned int i) {
  c[i]=0x00;
}

void resetchar(char *c) {
  c[0]=0x00;
}

unsigned int printhex(char *str) {
  unsigned int i;
  printf("HEX: ");
  for(i=0;i<strlen(str);i++) {
    printf("0x%x-",str[i]);
  }
  printf("\n");
  return 0;
}

void appchar(char *c, char a) {
  sprintf(c,"%s%c",c,a);
}

char *cctime(time_t a) {
  char *res;
  res=ctime(&a);
  delchar(res,'\r');
  delchar(res,'\n');
  return res;
}

/* Warning: This function needs to get free()'d after use! */
char *duration(long b) {
  long d,h,m,s,c; char *res; char tmp[60];
  c=b;
  d=(c/86400); c=(c%86400);
  h=(c/3600); c=(c%3600);
  m=(c/60); c=(c%60);
  s=c;
  sprintf(tmp,"%41dd %21dh %21dm %21ds",d,h,m,s);
  res=(char*)myalloc(strlen(tmp));
  strncpy(res,tmp,strlen(tmp));
  return res;
}

void durationcp(long b, char *res) {
  long d,h,m,s,c; char tmp[60];
  c=b;
  d=(c/86400); c=c%86400;
  h=(c/3600); c=c%3600;
  m=(c/60); c=c%60;
  s=c;
  sprintf(tmp,"%dd %02dh %02dm %02ds",d,h,m,s);
  strncpy(res,tmp,strlen(tmp));
  res[strlen(tmp)]=0x00;
}  

unsigned long duration2long(char *d) {
  unsigned long res; char tmpx[60]; char tmpy;
  strncpy(tmpx,d,strlen(d));
  dropcc(tmpx,(strlen(tmpx)-1));
  res=atol(tmpx);
  tmpy=d[strlen(d)-1];
  switch(tmpy) {
    case 's': return (res);
    case 'm': return (res*60);
    case 'h': return (res*3600);
    case 'd': return (res*86400);
    case 'w': return (res*604800);
    case 'M': return (res*2592000);
    case 'y': return (res*31104000);
  }
  return -1;
}

char *gettok(char *str, char *sep, int cs) {
  int i=1; char *p;
  p=(char*)strtok(str,sep);
  for(i=1;p!=NULL;i++) {
    if (cs==i) {
      return p;
    }
    p=(char*)strtok(NULL,sep);
  }
  return NULL;
}

/* Next proc is stolen from some library
   http://avrc.city.ac.uk/nethack/nethack-cxref-3.3.1/src/hacklib.c.src.html */
int strmatch(char *patrn, char *strng) {
  char s, p;
  /*
   :  Simple pattern matcher:  '*' matches 0 or more characters, '?' matches
   :  any single character.  Returns TRUE if 'strng' matches 'patrn'.
   */
 pmatch_top:
     s = *strng++;  p = *patrn++; /* get next chars and pre-advance */
     if (!p)                      /* end of pattern */
  return((s == '\0'));           /* matches iff end of string too */
     else if (p == '*')           /* wildcard reached */
  return(((!*patrn || strmatch(patrn, strng-1)) ? 1 : s ? strmatch(patrn-1, strng) : 0));
     else if (p != s && (p != '?' || !s))  /* check single character */
  return 0;           /* doesn't match */
     else                         /* return pmatch(patrn, strng); */
  goto pmatch_top;        /* optimize tail recursion */
}

/* This function is from my school-project "qbev2k" (C++)
   -- reworked version (bubblesort algorithm) */
void strsort(char *str) {
  int i,j; char s;
  for(i=strlen(str)-1;i>=1;i--) {
    for(j=0;j<strlen(str)-1;j++) {
      if (str[j] > str[j+1]) {
        s        = str[j];
        str[j]   = str[j+1];
        str[j+1] = s;
      }
    }
  }
}

/* And here a new, faster sort-algorithm
  -- the selection one. */
void strsortn(char *str) {
  int max,min,n,s; char a;
  max=strlen(str);
  for(n=0;n<max;n++) {
    min=n;
    for(s=n+1;s<max;s++) {
      if (str[s] < str[min]) min=s;
    } 
    a=str[n];
    str[n]=str[min];
    str[min]=a;
  }
}

void strcp(char *target, char *src) {
  strncpy(target,src,strlen(src));
  target[strlen(src)]=0x00;
}

void getmd5(char *src, char *tg) {
  int i=0; char res[40],sm[40];
  md5_state_t state;
  md5_byte_t digest[16];
  md5_init(&state);
  md5_append(&state, (const md5_byte_t *)src, strlen(src));
  md5_finish(&state, digest);
  for(res[0]=0x00,sm[0]=0x00,i=0; i<16; i++) {
    sprintf(sm,"%02x",digest[i]);
    strcat(res,sm);
  }
  res[32]=0x00;
  tg[0]=0x00;
  strcp(tg,res);
  if (global_debug>=2) printf("getmd5(): '%s' '%s'\n",src,res);
}

int is_null(char *ptr, int expr) {
  if (ptr == NULL || ptr[0]=='\0') {
    return 0;
  }
  assert(expr);
  return 1;
}
