/**********************************************************
 * ModServ2x - Modular IRC Services for advanced use.     *
 *   Based on the Undernet/Beware P10 Protocol.           *
 *   (C) 2004-2005 Christian Jurk (commx@commx.ws)        *
 *   Special thanks goes to:                              *
 *     Stephan Peijnik                                    *
 *     Linus Kimselius                                    *
 * ------------------------------------------------------ *
 * This program is released under the terms of the GNU    *
 * Public License (GPL). You'll find the License in the   *
 * docs directory shipped with this release.              *
 * ------------------------------------------------------ *
 * trusts.c                                               *
 * Connection count manipulation.                         *
 * 21/06/05 21:14 commx                                   *
 **********************************************************/
 
#include "include/main.h"
#include "include/ucdata.h"
#include "include/trusts.h"
#include "include/pgsql.h"
#include "include/strfunc.h"

trustgroup *istrusted(char *addr, users *u) {
  trusthost *a;
  for(a=th_list;a!=NULL;a=a->next) {
    if (!strncmp(a->addr,addr,16)) {
      return a->trustgroup;
    }
  }
  return NULL;
}

trustgroup *gettgbyid(unsigned int id) {
  trustgroup *a;
  for(a=tg_list;a!=NULL;a=a->next) {
    if (a->id == id) {
      return a;
    }
  }
  return NULL;
}

trustgroup *gettgbyname(char *name) {
  trustgroup *a;
  for(a=tg_list;a!=NULL;a=a->next) {
    if (!strcmp(a->name,name)) {
      return a;
    }
  }
  return NULL;
}

void loadtrusts(void) {
  PGresult *res,*rex; int i,j,k=0; trustgroup *tg,*tg2=NULL; trusthost *th;
  res=pg_query("SELECT contact,comment,connections,maxperident,forceident,expires,id,addedby,addedat,name FROM trustgroups;");
  if (pg_num_rows(res)<1) {
    PQclear(res);
    return;
  }
  for(i=0;i<pg_num_rows(res);i++) {
    tg=(trustgroup*)myalloc(sizeof(trustgroup));
    strcp(tg->name,PQgetvalue(res,i,9));
    strcp(tg->contact,PQgetvalue(res,i,0));
    strcp(tg->comment,PQgetvalue(res,i,1));
    strcp(tg->addedby,PQgetvalue(res,i,7));
    tg->id          = atoi(PQgetvalue(res,i,6));
    tg->connections = atoi(PQgetvalue(res,i,2));
    tg->maxperident = atoi(PQgetvalue(res,i,3));
    tg->expire      = atol(PQgetvalue(res,i,5));
    tg->addedat     = atol(PQgetvalue(res,i,8));
    tg->forceident  = (PQgetvalue(res,i,4)[0]=='t') ? 1 : 0;  
    tg->next=tg_list;
    tg_list=tg;
    rex=pg_query("SELECT address,maxusage,lastused FROM trusthosts WHERE trustgroup=%d;",tg->id);
    if (pg_num_rows(rex)<1) {
      PQclear(rex);
      continue;
    }
    for(j=0;j<pg_num_rows(rex);j++,k++) {
      th=(trusthost*)myalloc(sizeof(trusthost));
      strcp(th->addr,PQgetvalue(rex,j,0));
      th->trustgroup = tg;
      th->usage=getucbyhost(PQgetvalue(rex,j,0));
      th->lastused=atol(PQgetvalue(rex,j,2));
      th->maxusage=atol(PQgetvalue(rex,j,1));
      th->next=th_list;
      th_list=th;
    }
    PQclear(rex);
  }
  PQclear(res);
  if (global_debug) {
    flog("loadtrusts","Loaded %d trustgroup(s) with %d host(s).",i,k);
  }
  trusts_loaded = mTrue;
}

void savetrusts(void) {
  trustgroup *a; trusthost *b; int c=0;
  if (trusts_loaded != mTrue) return;
  for(a=tg_list;a!=NULL;a=a->next,c++) {
    pg_query("UPDATE trustgroups SET name='%s',contact='%s',comment='%s',maxperident=%d,forceident=%s,connections=%d,expires='%s' WHERE id=%d;",a->name,a->contact,a->comment,a->maxperident,(a->forceident==0) ? "FALSE" : "TRUE",a->connections,a->expire,a->id);
  }
  for(b=th_list;b!=NULL;b=b->next) {
    pg_query("UPDATE trusthosts SET maxusage=%d,lastused='%s' WHERE address='%s';",b->maxusage,b->lastused,b->addr);
  }
  if (global_debug && c>0) {
    flog("savetrusts","Updated %d trustgroup(s).",c);
  }
}

unsigned int getnewtgid() {
  PGresult *res; int r;
  res=pg_query("SELECT nextval('inc_tgid');");
  if (pg_num_rows(res)<1) {
    PQclear(res);
    return TG_ERR_NOID;
  }
  r=atoi(PQgetvalue(res,0,0));
  PQclear(res);
  return r;
}

void addtrustgroup(char *na, char *ow, char *ct, char *cm, char *ab, unsigned int mp, unsigned int co, unsigned int fi, time_t ex) {
  trustgroup *a; int id;
  id=getnewtgid();
  if (id == TG_ERR_NOID) {
    if (global_debug) {
      flog("addtrustgroup","ERROR: Cannot get a new trustgroup id (TG '%s')",na);
    }
    return;
  }
  a=(trustgroup*)myalloc(sizeof(trustgroup));
  strcp(a->name,na);
  strcp(a->contact,ct);
  strcp(a->comment,cm);
  strcp(a->addedby,ab);
  a->maxperident=mp;
  a->connections=co;
  a->forceident=fi;
  a->expire=ex;
  a->id=id;
  a->next=tg_list;
  tg_list=a;
  pg_query("INSERT INTO trustgroups (name,contact,comment,addedby,addedat,maxperident,connections,forceident,expires,id) VALUES ('%s','%s','%s','%s','%ld',%d,%d,%s,'%ld',%d);",a->name,a->contact,a->comment,a->addedby,time(NULL),mp,co,(fi==0) ? "FALSE" : "TRUE",ex,id);
  pg_query("INSERT INTO trustadmins (trustgroup,account,level) VALUES (%d,'%s','admin');",id,ow);
}

void deltrustgroup(unsigned int id) {
  trustgroup *a,*b=NULL; trusthost *c,*d=NULL; users *e,*f=NULL; int disconnected=0;
  for(a=tg_list;a!=NULL;a=a->next) {
    if (a->id==id) {
      if (b==NULL) {
        tg_list=a->next;
      } else {
        b->next=a->next;
      }
      free(a);
      return;
    }
    b=a;
  }
  for(c=th_list;c!=NULL;c=c->next) {
    if (c->trustgroup->id==id) {
      if (d==NULL) {
        th_list=c->next;
      } else {
        d->next=c->next;
      }
      free(c);
      c=th_list; continue;
    }
    d=c;
  }
  if (gettgoc(id)>mconfig_get_integer("maxclones",5)) {
    /* kill all users connected to that trustgroup */
    for(e=user_list;e!=NULL;e=e->next) {
      if (e->using_tg!=0 && e->using_tg==id) {
        if (f==NULL) {
          user_list=e->next;
        } else {
          f->next=e->next;
        }
        killuser(e->unum);
        send2up("%s D %s :%s (Trustgroup #%d was removed, you got disconnected.)",mconfig_get("numeric","JP"),e->unum,mconfig_get("servername","modserv.default.org"));
        free(e);
        disconnected++;
        e=user_list; continue;
      }
    }
  }
  pg_query("DELETE FROM trustgroups WHERE id=%d;",id);
  pg_query("DELETE FROM trusthosts WHERE trustgroup=%d;",id);
  pg_query("DELETE FROM trustadmins WHERE trustgroup=%d;",id);
  if (global_debug && disconnected>0) {
    slog("Trustgroup #%d was removed; %d user(s) are disconnected.",disconnected);
  }
}

unsigned int gettgoc(unsigned int id) {
  users *a; int using=0;
  for(a=user_list;a!=NULL;a=a->next) {
    if (a->using_tg!=0 && a->using_tg==id) {
      using++;
    }
  }
  return using;
}

void addtrust(char *addr, unsigned int id) {
  PGresult *res; trusthost *a; trustgroup *b;
  res=pg_query("SELECT * FROM trusthosts WHERE address='%s' AND trustgroup=%d;",addr,id);
  if (pg_num_rows(res)>0) {
    PQclear(res);
    flog("addtrust","ERROR: Cannot add new trust to Trustgroup #%d: Host already added (%s).",id,addr);
    return;
  }
  PQclear(res);
  b=gettgbyid(id);
  if (b==NULL) {
    flog("addtrust","ERROR: Cannot find Trustgroup #%d (gettgbyid).",id);
    return;
  }
  a=(trusthost*)myalloc(sizeof(trusthost));
  strcp(a->addr,addr);
  a->trustgroup=gettgbyid(id);
  a->usage=getucbyhost(a->addr);
  a->lastused=time(NULL);
  a->maxusage=0;
  a->next=th_list;
  th_list=a;
  pg_query("INSERT INTO trusthosts (address,trustgroup) VALUES ('%s',%d);",a->addr,a->trustgroup->id);
}

void deltrust(char *addr, unsigned int id) {
  trusthost *a,*b=NULL; users *c,*d=NULL;
  for(a=th_list;a!=NULL;a=a->next) {
    if (a->trustgroup->id==id && !strcmp(a->addr,addr)) {
      if (b==NULL) {
        th_list=a->next;
      } else {
        b->next=a->next;
      }
      free(b);
      break;
    }
    b=a;
  }
  pg_query("DELETE FROM trusthosts WHERE trustgroup=%d;",id);
  if (getucbyhost(addr) > mconfig_get_integer("maxclones",5)) {
    for(c=user_list;c!=NULL;c=c->next) {
      if (!strncmp(c->addr,addr,16)) {
        if (d==NULL) {
          user_list=c->next;
        } else {
          d->next=c->next;
        }
        free(c);
        c=user_list; continue;
      }
    }
  }
}

unsigned int getucbyhost(char *addr) {
  users *a; int count=0;
  for(a=user_list;a!=NULL;a=a->next) {
    if (!strncmp(a->addr,addr,16)) {
      count++;
    }
  }
  return count;
}

unsigned int getcctg(trustgroup *a) {
  trusthost *b; unsigned int c=0;
  for(b=th_list;b!=NULL;b=b->next) {
    if (b->trustgroup->id==a->id) {
      c+=b->usage;
    }
  }
  return c;
}
