/**********************************************************
 * ModServ2x - Modular IRC Services for advanced use.     *
 *   Based on the Undernet/Beware P10 Protocol.           *
 *   (C) 2004-2005 Christian Jurk (commx@commx.ws)        *
 *   Special thanks goes to:                              *
 *     Stephan Peijnik                                    *
 *     Linus Kimselius                                    *
 * ------------------------------------------------------ *
 * This program is released under the terms of the GNU    *
 * Public License (GPL). You'll find the License in the   *
 * docs directory shipped with this release.              *
 * ------------------------------------------------------ *
 * yalloc.c                                               *
 * Extended memstr functions.                             *
 * 21/04/05 12;05 commx                                   *
 **********************************************************/
   
#include "include/yalloc.h"
#include "include/yalloc2.h" /* yalloc v2 */

void *myalloc(size_t size_bytes) {
  void *ptr; 
  if ((ptr=malloc(size_bytes))==NULL) {
    flog("myalloc","Fatal error: Cannot allocate memory (%d byte%s), cancelling request.",size_bytes,(size_bytes>1) ? "s" : "");
    return NULL;
  }
  bytecounter+=size_bytes;
  return ptr;
}

void myfree(void *memptr) {
  assert(memptr);
  size_t siz=sizeof(memptr);
  free(memptr);
  bytecounter-=siz;
}

void ypcpy(char *target, char *source, int haveptr) {
  if (haveptr && ((target=(char*)myalloc(strlen(source)+1))==NULL)) return;
  strncpy(target,source,strlen(source));
  target[strlen(source)]=0x00;
}

yalloc *yalloc_alloc(unsigned int i, size_t s) {
  void *memptr; yalloc *y;
  y=yalloc_yptr(i);
  if (!y) {
    memptr=malloc(s);
    if (memptr==NULL) {
      slog("Fatal error: Cannot allocate memory (%d bytes), id %d -- cancelling allocation.",s,i);
      return;
    }
    y=(yalloc *)(memptr);
    if (y==NULL) {
      slog("Fatal error: Cannot allocate memory (%d bytes), id %d -- yalloc points to NULL?!",s,i);
      return;
    }
    y->iptr=(unsigned int)yalloc_getid();
    y->bytes=s;
    y->mptr=memptr;
  } else {
    memptr=malloc(s);
    if (memptr==NULL) {
      slog("Fatal error: Cannot allocate memory (%d bytes), id %d -- cancelling allocation.",s,i);
      return;
    }
    y=yalloc_yptr(i);
    if (!y) {
      slog("Fatal error: Cannot allocate memory (%d bytes), id %d -- unknown entry?!",s,i);
      return;
    }
    y->bytes+=s;
    y->mptr=memptr; /* set pointer to new allocated memory. old one will still exist, but is lost somewhere else */
  }
}

yalloc *yalloc_yptr(unsigned int i) {
  yalloc *y;
  for(y=yalloc_list;y!=NULL;y=(void*)y->next) {
    if (y->iptr==i) {
      return y;
    }
  }
  return 0;
}

yalloc *yalloc_free(unsigned int i) {
  yalloc *y;
  y=yalloc_yptr(i);
  if (!y) {
    slog("Fatal error: Cannot free memory because i cannot find the item. id %d.",i);
    return;
  }
  free(y->mptr);
  free(y);
}

unsigned int yalloc_getid() {
  unsigned int a; yalloc *y;
  for(a=0,y=yalloc_list;y!=NULL;a++);
  y=yalloc_yptr(a);
  if (y!=0) { yalloc_getid(); }
  return a;
}

char *yalloc_new(size_t s) {
  char *d;
  d=(char*)(malloc(s));
  if (d==NULL) {
    slog("** Fatal Error: yalloc_new(): Cannot allocate memory (%d bytes)!",s);
    return;
  }
  return d;
}

char *yalloc_nc(char *str) {
  char *template; unsigned int i,l,d=1;
  if (str[0]=='\001') return NULL;
  for(i=0;i<strlen(str);i++) {
    if (str[i]=='\0') d++;
  }
  l=strlen(str)+d;
  if (str==NULL || str[0]=='\0') return NULL;
  if ((template=(char*)(malloc(l)))==NULL) {
    slog("** FATAL: Cannot allocate memory for '%s': %d chars.",str,l);
    exit(-1);
  }
  strncpy(template,str,l);
  dropcc(template,l);
  return template;
}

/* Implementation of yalloc v2 - see yalloc2.h */
